/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FirewallDialog.cc,v 1.134 2002/05/07 02:08:15 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "fwbuilder/Firewall.hh"
#include "fwbuilder/FWOptions.hh"
#include "fwbuilder/Management.hh"
#include "fwbuilder/Policy.hh"
#include "fwbuilder/InterfacePolicy.hh"
#include "fwbuilder/NAT.hh"
#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/snmp.hh"
#include "fwbuilder/dns.hh"

#include "FirewallDialog.hh"
#include "GenericBackgroundOpDialog.hh"
#include "MessageDialog.hh"
#include "PolicyDialog.hh"
#include "IPAddressWidget.hh"
#include "helpers.hh"
#include "FileSel.hh"
#include "Preferences.hh"
#include "main_window.hh"

#include <gtk--.h>

#define GENERAL_NOTEBOOK_PAGE          0
#define INTERFACES_NOTEBOOK_PAGE       1
#define SNMP_NOTEBOOK_PAGE             2
#define COMPILER_NOTEBOOK_PAGE         3

#define NO_FIREWALL_NOTEBOOK_PAGE      4
#define IPFILTER_NOTEBOOK_PAGE         5
#define IPTABLES_NOTEBOOK_PAGE         6
#define CISCO_NOTEBOOK_PAGE            7
#define PIX_NOTEBOOK_PAGE              8
#define PF_NOTEBOOK_PAGE               9
#define IPF_NOTEBOOK_PAGE              10

#define LAST_PLATFORM_PAGE             IPF_NOTEBOOK_PAGE

#define NO_OS_NOTEBOOK_PAGE            11
#define LINUX24_NOTEBOOK_PAGE          12
#define PIX_OS_NOTEBOOK_PAGE           13
#define OPENBSD_OS_NOTEBOOK_PAGE       14
#define FREEBSD_OS_NOTEBOOK_PAGE     15
#define SOLARIS_OS_NOTEBOOK_PAGE       16

#define LAST_OS_PAGE                   SOLARIS_OS_NOTEBOOK_PAGE

using namespace libfwbuilder;



FirewallDialog::FirewallDialog(FWObject *obj) 
{ 
    set_name("firewall_dialog");

    interface_policy_list=NULL;

    object=obj;

    options=NULL;

    fw_addr=manage(new IPAddressWidget());
    table16->attach(*fw_addr, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
    fw_addr->changed.connect(SigC::slot(this,&FirewallDialog::on_changed));

    firewall_dialog_notebook->switch_page.connect(SigC::slot(this, 
					   &FirewallDialog::on_switch_page));


    if_list->select_row.connect(SigC::slot(this, 
				  &FirewallDialog::on_if_list_select_row));



    network_tab_txt2->set_line_wrap(true);

    log_levels[""       ]="        "  ;
    log_levels["alert"  ]="1: alert"  ;
    log_levels["crit"   ]="2: crit"   ;
    log_levels["error"  ]="3: error"  ;
    log_levels["warning"]="4: warning";
    log_levels["notice" ]="5: notice" ;
    log_levels["info"   ]="6: info"   ;
    log_levels["debug"  ]="7: debug"  ;

    log_facilities[""    ]="      "  ;
    log_facilities["16"  ]="LOCAL0"  ;
    log_facilities["17"  ]="LOCAL1"  ;
    log_facilities["18"  ]="LOCAL2"  ;
    log_facilities["19"  ]="LOCAL3"  ;
    log_facilities["20"  ]="LOCAL4"  ;
    log_facilities["21"  ]="LOCAL5"  ;
    log_facilities["22"  ]="LOCAL6"  ;
    log_facilities["23"  ]="LOCAL7"  ;

    log_facilities_full[""         ]="             "  ;
    log_facilities_full["kern"     ]=" 0: kern     "  ;
    log_facilities_full["user"     ]=" 1: user     "  ;
    log_facilities_full["mail"     ]=" 2: mail     "  ;
    log_facilities_full["daemon"   ]=" 3: daemon   "  ;
    log_facilities_full["auth"     ]=" 4: auth     "  ;
    log_facilities_full["syslog"   ]=" 5: syslog   "  ;
    log_facilities_full["lpr"      ]=" 6: lpr      "  ;
    log_facilities_full["news"     ]=" 7: news     "  ;
    log_facilities_full["uucp"     ]=" 8: uucp     "  ;
    log_facilities_full["cron"     ]=" 9: cron     "  ;
    log_facilities_full["authpriv" ]="10: authpriv "  ;
    log_facilities_full["ftp"      ]="11: ftp      "  ;
    log_facilities_full["local0"   ]="16: local0   "  ;
    log_facilities_full["local1"   ]="17: local1   "  ;
    log_facilities_full["local2"   ]="18: local2   "  ;
    log_facilities_full["local3"   ]="19: local3   "  ;
    log_facilities_full["local4"   ]="20: local4   "  ;
    log_facilities_full["local5"   ]="21: local5   "  ;
    log_facilities_full["local6"   ]="22: local6   "  ;
    log_facilities_full["local7"   ]="23: local7   "  ;




//    host_os_menu["Unknown"]   = DialogPlugin::magic_empty_option;
//    host_os_menu["Linux 2.4"] = "linux24";
//    host_os_menu["OpenBSD"]   = "OpenBSD";
//    host_os_menu["Cisco PIX"] = "pix";

    host_os_menu=Resources::global_res->getOS();

    host_os->set_menu(host_os_menu);
    host_os->changed.connect(SigC::slot(this,&FirewallDialog::on_host_os_changed));
    host_os->show_all();


    platforms=Resources::global_res->getPlatforms();

    fw_platform->set_menu(platforms);
    fw_platform->changed.connect(SigC::slot(this,&FirewallDialog::on_fw_platform_changed));
    fw_platform->show_all();

//    on_off_menu["On"]="1";
//    on_off_menu["Off"]="0";
//    on_off_menu["No change"]=DialogPlugin::magic_empty_option;

    on_off_menu["1"]="On";
    on_off_menu["0"]="Off";
    on_off_menu[DialogPlugin::magic_empty_option]="No change";


/***********************************/
					      		                      
    object_parameters.push_back( DialogOption( fw_name          , "name"                 ) );
    object_parameters.push_back( DialogOption( comment          , "comment"              ) );
    object_parameters.push_back( DialogOption( fw_addr          , "address"              ) );
    object_parameters.push_back( DialogOption( fw_platform      , "platform"             ) );
    object_parameters.push_back( DialogOption( host_os          , "host_OS"              ) );
    object_parameters.push_back( DialogOption( fw_version       , "version"              ) );


    common_options.push_back(    DialogOption( dyn_addr         , "dyn_addr"             ) );

    common_options.push_back(    DialogOption( snmp_description , "snmp_description"     ) );
    common_options.push_back(    DialogOption( snmp_location    , "snmp_location"        ) );
    common_options.push_back(    DialogOption( snmp_contact     , "snmp_contact"         ) );

    common_options.push_back(    DialogOption( cmdline          , "cmdline"              ) );
    common_options.push_back(    DialogOption( compiler         , "compiler"             ) );

/***********************************/

    init_iptables();
    init_ipfilter();
    init_pix();
    init_pf();
    init_ipf();

    init_linux24();
    init_pix_os();
    init_openbsd();
    init_freebsd();
    init_solaris();

    mgmt_txt1->set_line_wrap(true);
    mgmt_txt2->set_line_wrap(true);
    mgmt_txt3->set_line_wrap(true);

}

FirewallDialog::~FirewallDialog() 
{ 
}

void FirewallDialog::on_mgmt_use_install_script_clicked()
{
#ifndef __MINGW32__
    disableFWBDoptions();
    on_changed();
#endif
}

void FirewallDialog::on_mgmt_use_fwbd_clicked()
{
    enableFWBDoptions();
    on_changed();
}

void FirewallDialog::enableFWBDoptions()
{
#ifdef HAVE_LIBSSL
    mgmt_fwbd_port->set_sensitive(true);
    mgmt_keys->set_sensitive(true);
    mgmt_fw_key->set_sensitive(true);
    mgmt_clear_fw_key->set_sensitive(true);

    inst_script->set_sensitive(false);
    inst_cmdline->set_sensitive(false);
#endif
}

void FirewallDialog::disableFWBDoptions()
{
    mgmt_fwbd_port->set_sensitive(false);
    mgmt_keys->set_sensitive(false);
    mgmt_fw_key->set_sensitive(false);
    mgmt_clear_fw_key->set_sensitive(false);

    inst_script->set_sensitive(true);
    inst_cmdline->set_sensitive(true);
}

void FirewallDialog::hideAllFirewallTabs()
{
    firewall_dialog_notebook->get_nth_page(NO_FIREWALL_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPFILTER_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPTABLES_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(CISCO_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(PIX_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(PF_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(IPF_NOTEBOOK_PAGE)->hide();
}

void FirewallDialog::hideAllOSTabs()
{
    firewall_dialog_notebook->get_nth_page(NO_OS_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(LINUX24_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(PIX_OS_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(OPENBSD_OS_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(FREEBSD_OS_NOTEBOOK_PAGE)->hide();
    firewall_dialog_notebook->get_nth_page(SOLARIS_OS_NOTEBOOK_PAGE)->hide();
}

void FirewallDialog::showFirewallTab(guint tab_no)
{
    assert (tab_no>=NO_FIREWALL_NOTEBOOK_PAGE && tab_no<=LAST_PLATFORM_PAGE);

    hideAllFirewallTabs();
    firewall_dialog_notebook->get_nth_page(tab_no)->show();
}

void FirewallDialog::showOSTab(guint tab_no)
{
    assert (tab_no>=NO_OS_NOTEBOOK_PAGE && tab_no<=LAST_OS_PAGE);

    hideAllOSTabs();
    firewall_dialog_notebook->get_nth_page(tab_no)->show();
}

void FirewallDialog::showChosenFirewallTab()
{
    string new_platform=fw_platform->get_value();

    if ( new_platform=="ipfilter") { showFirewallTab(IPFILTER_NOTEBOOK_PAGE);
    } else 
	if ( new_platform=="iptables") { showFirewallTab(IPTABLES_NOTEBOOK_PAGE);
	} else
	    if ( new_platform=="cisco") { showFirewallTab(CISCO_NOTEBOOK_PAGE);
	    } else
		if ( new_platform=="pix") { showFirewallTab(PIX_NOTEBOOK_PAGE);
		} else  
		    if ( new_platform=="pf") { showFirewallTab(PF_NOTEBOOK_PAGE);
		    } else  
			if ( new_platform=="ipf") { showFirewallTab(IPF_NOTEBOOK_PAGE);
			} else  
			    showFirewallTab(NO_FIREWALL_NOTEBOOK_PAGE);
}

void FirewallDialog::showChosenOSTab()
{
    string new_os=host_os->get_value();

    if ( new_os=="linux24") { showOSTab(LINUX24_NOTEBOOK_PAGE);
    } else
	if ( new_os=="pix") { showOSTab(PIX_OS_NOTEBOOK_PAGE);
	} else 
	    if ( new_os=="openbsd") { showOSTab(OPENBSD_OS_NOTEBOOK_PAGE);
	    } else  
		if ( new_os=="freebsd") { showOSTab(FREEBSD_OS_NOTEBOOK_PAGE);
		} else  
		    if ( new_os=="solaris") { showOSTab(SOLARIS_OS_NOTEBOOK_PAGE);
		    } else  
			showOSTab(NO_OS_NOTEBOOK_PAGE);



//    if (new_os=="linux24") {
//	enableFWBDoptions();
//	mgmt_use_fwbd->set_active(true);
//    }
}


/**
 *   this callback is called when user changes host OS
 */
void FirewallDialog::on_host_os_changed()
{
/* hide all host OS related pages */
    hideAllOSTabs();
    showChosenOSTab();

    string new_os=host_os->get_value();

    saveOptionMenu(object, host_os , "host_OS");
    Firewall::cast(object)->setDefaults(); 

    if (new_os=="linux24")	loadPageOptions(options,linux24_options );
    if (new_os=="pix")	        loadPageOptions(options,pix_os_options );
    if (new_os=="openbsd")	loadPageOptions(options,openbsd_options );
    if (new_os=="freebsd")	loadPageOptions(options,freebsd_options );
    if (new_os=="solaris")	loadPageOptions(options,solaris_options );

    on_changed();
}

/**
 *   this callback is called when user changes firewall platform
 */
void FirewallDialog::on_fw_platform_changed()
{
    hideAllFirewallTabs();
    showChosenFirewallTab();

    string new_platform=fw_platform->get_value();

    saveOptionMenu(object, fw_platform , "platform");
    Firewall::cast(object)->setDefaults();  // set default parameters for chosen platform

    if (new_platform=="ipfilter")  loadPageOptions(options,ipfilter_options );
    if (new_platform=="iptables")  loadPageOptions(options,iptables_options );
    if (new_platform=="pix")       loadPageOptions(options,pix_options );
    if (new_platform=="openbsd")   loadPageOptions(options,openbsd_options );
    if (new_platform=="freebsd") loadPageOptions(options,freebsd_options );
    if (new_platform=="solaris")   loadPageOptions(options,solaris_options );

    on_changed();
}

void FirewallDialog::on_switch_page(Gtk::Notebook_Helpers::Page* p0,guint p1)
{
    // do nothing if dialog is not visible
    // it turns out, "switch_page" signal gets invoked 
    // when notebook pages are destroyed
    if(!is_visible()) 
        return;

    updateMainMenu();
    switchToPage(p1);
}

string  FirewallDialog::getCertDescString(int no, const string& d)
{
    char buf[32];
    sprintf(buf,"%d: ",no);
    return  string(buf)+ d;
}

void    FirewallDialog::fillListOfCertificates()
{
    Management *mgmt=(Firewall::cast(object))->getManagementObject();
    assert(mgmt!=NULL);

    map<string, pair<string, string> > certs= Preferences::global_prefs->getCerificates();
    map<string, pair<string, string> >::iterator i;
    vector<string>     cert_list;

    string fw_cert_id=mgmt->getFWBDManagement()->getIdentityId();
    string fw_cert_desc=certs[fw_cert_id].first;
    int n=0;
    for(i=certs.begin(); i!=certs.end(); i++,n++) {	
	if ( ! (*i).first.empty() )
	    cert_list.push_back( getCertDescString(n,(*i).second.first ) );
    }

    mgmt_keys->set_popdown_strings( cert_list );
    mgmt_keys->get_entry()->set_text( fw_cert_desc );
}

/*
 * an (unsuccessfull) attempt to add ID's of certificates as
 * user_data. This won't work because widget Gtk::Combo returns data
 * as a string in Gtk::Entry, not as an item in the list, so we can't
 * really pull user_data from there

    Gtk::List_Helpers::ItemList           ilist=mgmt_keys->get_list()->items();
    Gtk::List_Helpers::ItemList::iterator j;

    for(i=certs.begin(), j=ilist.begin(); 
	i!=certs.end() && j!=ilist.end(); 
	i++,j++) {
	(*j)->set_data_full("cert_id", new string( (*i).first ), &_cert_list_destroy_string );
	cerr << "*** " << (*i).first << "   " << (*i).second.first << endl;
    }

}

static void _cert_list_destroy_string(gpointer data)
{
    string *sp= (string*)data;
    delete sp;
}
*/


/*
 *  Regarding FirewallOptions *options
 *
 *  Unfortunately we have to find this objects every time we load data
 *  into dialog because it may change. In particular, it changes when
 *  user clicks "Undo". This happens because dialog actually works
 *  with a copy of original object in ScratchPad. When user clicks
 *  Undo this copy is overriden with data from original; when this
 *  happens, all child objects are recreated.  (vk)
 */

/**
 *  This method is called to load data from the object into dialog
 *  This happens at the very beginning and when user clicks "Undo"
 *  (vk)
 */
void FirewallDialog::wrk2dlg()
{
    options=(Firewall::cast(object))->getOptionsObject();
    assert(options!=NULL);

    Management *mgmt=(Firewall::cast(object))->getManagementObject();
    assert(mgmt!=NULL);

    fw_name->grab_focus();


#ifndef HAVE_LIBSNMP
    button4->set_sensitive(false);
    button16->set_sensitive(false);
#endif


    loadPageOptions(object, object_parameters );

    loadPageOptions(options, common_options );

    showInterfaces();

    showChosenFirewallTab();
    showChosenOSTab();

    loadPageOptions(options,  ipfilter_options );
    loadPageOptions(options,  iptables_options );
    loadPageOptions(options,  pix_options      );
    loadPageOptions(options,  pf_options       );
    loadPageOptions(options,  ipf_options      );

    loadPageOptions(options,  linux24_options  );
    loadPageOptions(options,  pix_os_options   );
    loadPageOptions(options,  openbsd_options  );
    loadPageOptions(options,  freebsd_options);
    loadPageOptions(options,  solaris_options);


/*********************  data for fwbd and install script **************/
    FWBDManagement      *fwbdm = mgmt->getFWBDManagement();
    PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
    SNMPManagement      *snmpm = mgmt->getSNMPManagement();
    
    snmp_r_community->set_text(snmpm->getReadCommunity());
    snmp_w_community->set_text(snmpm->getWriteCommunity());

    inst_script->set_text(  pis->getCommand() );
    inst_cmdline->set_text( pis->getArguments() );

#ifdef HAVE_LIBSSL

    int port=fwbdm->getPort();
    if (port==-1) 
	port= Resources::global_res->getResourceInt("/FWBuilderResources/FWBD/port");
    mgmt_fwbd_port->set_value( port );

    fillListOfCertificates();

    const Key *key=fwbdm->getPublicKey();
    if (key) mgmt_fw_key->set_text( key->getFingerprint() );

#ifdef __MINGW32__
    mgmt_use_fwbd->set_active(true);
#else
    if (pis->isEnabled()) mgmt_use_install_script->set_active(true);
    else                  mgmt_use_fwbd->set_active(true);
#endif

#else
    mgmt_use_fwbd->set_sensitive(false);
    mgmt_use_install_script->set_active(true);

    disableFWBDoptions();
/*
    mgmt_keys->hide();
    mgmt_fwbd_port->hide();
    mgmt_fw_key->hide();
    mgmt_clear_fw_key->hide();
    mgmt_txt1->hide();
    mgmt_txt2->hide();
    mgmt_txt3->hide();

    mgmt_txt1->set_text("Installation via fwbd daemon is disabled because program has been compiled without SSL support");
    mgmt_txt1->show();
*/
#endif
}

/**
 *    This method is called to copy dialog data into the object (when
 *    user clicks "Apply" ) (vk)
 */
bool FirewallDialog::dlg2wrk()
{
    Management *mgmt=(Firewall::cast(object))->getManagementObject();
    assert(mgmt!=NULL);

/*
 *  check if name is valid
 */
    if ( ! checkObjectName(fw_name->get_text()) ) return(false);
    if ( ! dyn_addr->get_active() &&
	 ! checkIPaddress(fw_addr->getAsString()) ) return(false);

    savePageOptions(object, object_parameters );

    savePageOptions(options, common_options );

    if (string(object->getStr("platform"))=="ipfilter") 
	savePageOptions(options,  ipfilter_options );
    if (string(object->getStr("platform"))=="iptables") 
	savePageOptions(options,  iptables_options );
    if (string(object->getStr("platform"))=="pix") 
	savePageOptions(options,  pix_options );
    if (string(object->getStr("platform"))=="pf") 
	savePageOptions(options,  pf_options );
    if (string(object->getStr("platform"))=="ipf") 
	savePageOptions(options,  ipf_options );

    if (string(object->getStr("host_OS"))=="linux24") 
	savePageOptions(options,  linux24_options );
    if (string(object->getStr("host_OS"))=="pix") 
	savePageOptions(options,  pix_os_options );
    if (string(object->getStr("host_OS"))=="openbsd") 
	savePageOptions(options,  openbsd_options );
    if (string(object->getStr("host_OS"))=="freebsd") 
	savePageOptions(options,  freebsd_options );
    if (string(object->getStr("host_OS"))=="solaris") 
	savePageOptions(options,  solaris_options );


/*********************  management data *******************8*/

    FWBDManagement      *fwbdm = mgmt->getFWBDManagement();
    PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();
    SNMPManagement      *snmpm = mgmt->getSNMPManagement();

    mgmt->setAddress( fw_addr->getAsIPAddress() );

    snmpm->setReadCommunity( snmp_r_community->get_text() );
    snmpm->setWriteCommunity( snmp_w_community->get_text() );

    fwbdm->setPort( mgmt_fwbd_port->get_value_as_int() );

    map<string, pair<string, string> > certs= Preferences::global_prefs->getCerificates();
    map<string, pair<string, string> >::iterator i;

    string cert=mgmt_keys->get_entry()->get_text();
    int n=0;
    for(i=certs.begin(); i!=certs.end(); i++,n++) 
	if ( ! (*i).first.empty() && cert == getCertDescString(n , (*i).second.first ) ) {
	    fwbdm->setIdentityId( (*i).first );
	    break;
	}

    pis->setCommand( inst_script->get_text() );
    pis->setArguments( inst_cmdline->get_text() );

    pis->setEnabled( mgmt_use_install_script->get_active() );
    fwbdm->setEnabled( mgmt_use_fwbd->get_active() );

    return(true);
}

void FirewallDialog::on_mgmt_clear_fw_key_clicked()
{
    Management *mgmt=(Firewall::cast(object))->getManagementObject();
    assert(mgmt!=NULL);

    mgmt->getFWBDManagement()->removePublicKey();
    mgmt_fw_key->set_text("");
    on_changed();
}

void FirewallDialog::on_changed()
{
    data_changed_flag(true);
}

void FirewallDialog::on_snmp_get_descr_released()
{
#ifdef HAVE_LIBSNMP

    string addr;
    string rcomm=snmp_r_community->get_text();

    if ( dyn_addr->get_active() ) {
	addr=fw_name->get_text();
    } else {
	if ( ! checkIPaddress(fw_addr->getAsString()) ) return;
	addr=fw_addr->getAsString();
    }


    if(rcomm.empty()) 
    {
	MessageDialog::Error("No SNMP community string defined");
	return;
    }

    int t=Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPTimeout", -1);

    SNMP_sysdesc_query *q=new SNMP_sysdesc_query();

    q->init(addr , rcomm,
	    Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPRetries",  SNMP_DEFAULT_RETRIES),
	    t==-1?SNMP_DEFAULT_TIMEOUT:(1000000L*t)
    );


    GenericBackgroundOpDialog bdisp(q);
    
    try
    {
        bdisp.execute();
        snmp_description->set_point(0);
        snmp_description->forward_delete( snmp_description->get_length() );
        snmp_description->insert( q->getDescr());
        
        snmp_location->set_text(q->getLocation());
        snmp_contact->set_text(q->getContact());

        data_changed_flag(true);
    } catch(const FWException &ex)
    {
        //do nothing
    }

    bdisp.disconnect();
    delete q;

#endif
}

void FirewallDialog::on_find_compiler_clicked()
{   
    FileSel *fs=new FileSel("Find compiler",
                            Preferences::global_prefs->getWdir(),
                            "");
    string cmp=fs->run();
    delete fs;
    if(cmp!="")
	compiler->set_text( cmp );
}

void FirewallDialog::on_find_install_script_clicked()
{   
    FileSel *fs=new FileSel("Find install script",
                            Preferences::global_prefs->getWdir(),
                            "");
    string ins=fs->run();
    delete fs;
    if(ins!="")
	inst_script->set_text( ins );
}

void FirewallDialog::on_dyn_addr_toggled()
{   
    fw_addr->set_sensitive( ! dyn_addr->get_active() );
}

void FirewallDialog::on_dns_lookup()
{
    vector<IPAddress> addr;

    try {
	addr=DNS::getHostByName( fw_name->get_text() );
    } catch (FWException &e)
    {
	MessageDialog::Error( e.toString() );
	return;
    }

    fw_addr->setAddress( addr.front() );
    data_changed_flag(true);
}

void FirewallDialog::switchToPage(guint page_n)
{
    Gtk::MenuBar *main_menubar=main_w->getMainMenuBar();
    Gtk::Widget *itm;

    if (page_n==INTERFACES_NOTEBOOK_PAGE) {

	itm=find_widget("insert_top",main_menubar);
	if (itm) itm->set_sensitive(true);

	itm=find_widget("append_bottom",main_menubar);
	if (itm) itm->set_sensitive(true);

	itm=find_widget("add_after",main_menubar);
	if (itm) itm->set_sensitive(true);

	itm=find_widget("remove_current",main_menubar);
	if (itm) itm->set_sensitive(true);

	itm=find_widget("move_up",main_menubar);
	if (itm) itm->set_sensitive(true);

	itm=find_widget("move_down",main_menubar);
	if (itm) itm->set_sensitive(true);
    }

}


/**
 *  enable and disable menu items which make sence only for this object
 */
void FirewallDialog::updateMainMenu()
{
    Gtk::MenuBar *main_menubar=main_w->getMainMenuBar();
    Gtk::Widget *itm;

    DialogPlugin::updateMainMenu();

    itm=find_widget("policy",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("rules_druid",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("compile_policy",main_menubar);
    if (itm) itm->set_sensitive(true);

    Management *mgmt=(Firewall::cast(object))->getManagementObject();
    assert(mgmt!=NULL);
    PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();

    itm=find_widget("install_policy",main_menubar);
    if (itm) itm->set_sensitive( pis->isEnabled() );

}



