/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FirewallDialog_Interfaces.cc,v 1.9 2002/05/07 02:08:15 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "fwbuilder/Firewall.hh"
#include "fwbuilder/FWOptions.hh"
#include "fwbuilder/Policy.hh"
#include "fwbuilder/InterfacePolicy.hh"
#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/snmp.hh"

#include "FirewallDialog.hh"
#include "InterfaceDialog.hh"
#include "IPAddressWidget.hh"

#include "GenericBackgroundOpDialog.hh"
#include "MessageDialog.hh"
#include "PolicyList.hh"
#include "helpers.hh"
#include "platforms.hh"

#include "Preferences.hh"

using namespace libfwbuilder;

void FirewallDialog::showInterfaces(const string &activate_id)
{
    if_list->freeze();
    if_list->clear();

    list<FWObject*>::iterator m;
    gint row, activate_row;

    if ( ((Gtk::Bin*)interface_policy_vprt)->get_child()!=NULL )
	((Gtk::Bin*)interface_policy_vprt)->remove();

/*
 * TODO:  these settings should come from capabilities database in resources
 */
    if (fw_platform->get_value()=="pix") {
	if_list->set_show_security_level(true);
	if_list->set_show_dyn(false);
    } else {
	if_list->set_show_security_level(false);
	if_list->set_show_dyn(true);
    }

    activate_row=0;
    for (row=0,m=((Host*)object)->begin(); m!=((Host*)object)->end(); ++m) {

	if(Interface::isA((*m))) {
	    if_list->addInterface( Interface::cast(*m) );
	    if ( (*m)->getId()==activate_id) activate_row=row;
	    ++row;
	}
    }
    if_list->thaw();
    if_list->show_all();

    if_list->row(activate_row).select();
}

void FirewallDialog::showInterfacePolicy( Interface* intf )
{
    InterfacePolicy *ipol;

    if (intf) {
	ipol=InterfacePolicy::cast(
	    intf->getFirstByType( InterfacePolicy::TYPENAME ) );
	if (ipol==NULL) {
	    ipol=new InterfacePolicy();
	    intf->add(ipol);
	}

//	DialogPlugin *dlg= manage( ipol->CreateDialog() );

	if ( ((Gtk::Bin*)interface_policy_vprt)->get_child()!=NULL )
	    ((Gtk::Bin*)interface_policy_vprt)->remove();

	interface_policy_list=manage(new InterfacePolicyList(ipol));
	interface_policy_vprt->add( *interface_policy_list );
	interface_policy_list->build();
	interface_policy_list->show_all();

	interface_policy_frame->set_label( 
	    string("Policy attached to interface ")+intf->getName() );


    }
}

void FirewallDialog::on_new_if_clicked()
{ 
    bool res;

    Interface *intf=new Interface();

    InterfaceDialog *id = new InterfaceDialog(intf);

    if (isUsingNetZone(Firewall::cast(object))) {
	id->useSecurityLevel();
	id->useNetworkZone();
    } else
	id->useExtCheckbox();

    res=id->run();

    delete id;

    if (res) {
	if ( (Firewall::cast(object))->Appropriate(intf) ) 
	    object->add(intf);

	intf->add( new InterfacePolicy() );

	showInterfaces();
	data_changed_flag(true);
    } else
	delete intf;
}

void FirewallDialog::on_del_if_clicked()
{   
    Interface *intf=getSelectedInterface();
    if (intf) {
	object->remove(intf);
	showInterfaces();
	data_changed_flag(true);
    }
}


void FirewallDialog::on_edit_clicked()
{   
    bool res;

    Interface *intf=getSelectedInterface();
    if(intf) 
    {
	InterfaceDialog *id = new InterfaceDialog( intf );

	if (isUsingNetZone(Firewall::cast(object))) {
	    id->useSecurityLevel();
	    id->useNetworkZone();
	} else
	    id->useExtCheckbox();


	res=id->run();
	delete id;
	if(res) 
        {
	    showInterfaces( intf->getId() );
	    data_changed_flag(true);
	}
    }
}

void FirewallDialog::on_snmp_get_clicked()
{
}

void FirewallDialog::on_snmp_get_released()
{   
#ifdef HAVE_LIBSNMP
    string addr;
    string rcomm=snmp_r_community->get_text();

    if ( dyn_addr->get_active() ) {
	addr=fw_name->get_text();
    } else {
	if ( ! checkIPaddress(fw_addr->getAsString()) ) return;
	addr=fw_addr->getAsString();
    }

    if ( rcomm.empty() ) {
	MessageDialog::Error("No SNMP community string defined");
	return;
    }

    FWObject *o;
    while ( (o=object->getFirstByType(Interface::TYPENAME))!=NULL )
	object->remove(o);

    int t=Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPTimeout", -1);

    SNMP_interface_query *q=new SNMP_interface_query();
    q->init(addr,  rcomm,
	    Preferences::global_prefs->getOptInt("/FWBuilderPreferences/Network/SNMPRetries",  SNMP_DEFAULT_RETRIES),
	    t==-1?SNMP_DEFAULT_TIMEOUT:(1000000L*t)
    );

    GenericBackgroundOpDialog bdisp(q);
    
    try
    {
        bdisp.execute();
        
        const map<int, Interface> &intf = q->getInterfaces();
        for(map<int, Interface>::const_iterator i=intf.begin();i!=intf.end(); ++i)
        {
            Interface *ifs=new Interface((*i).second);
            if((Firewall::cast(object))->Appropriate(ifs))
            {
                ifs->add(new InterfacePolicy());
                object->add( ifs );
            }
        }
        showInterfaces();
        data_changed_flag(true);
    } catch(const FWException &ex)
    {
        //do nothing
    }

    bdisp.disconnect();
    delete q;

#endif
}

void FirewallDialog::on_if_list_select_row(gint row, 
					   gint column, 
					   GdkEvent *event)
{   
    Interface *intf;

    string id=if_list->getIdByRow(row);
    if(!id.empty()) 
    {
	intf = Interface::cast( object->getById( id ) );
	if(intf) 
            showInterfacePolicy( intf );
    }
}

Interface* FirewallDialog::getInterfaceByRow(int row)
{
    Interface *intf=NULL;
    string id=if_list->getIdByRow(row);
    if(!id.empty()) 
	intf = Interface::cast( object->getById( id ) );

    return intf;
}

Interface* FirewallDialog::getSelectedInterface()
{
    Interface *intf=NULL;

    string id=if_list->getSelectedInterfaceId();
    if(!id.empty()) 
	intf = Interface::cast( object->getById( id ) );

    return intf;
}

void FirewallDialog::on_add_rule_clicked()
{   
    interface_policy_list->appendRuleAtBottom();
    return;

    Interface *intf=getSelectedInterface();
    if (intf) {
	InterfacePolicy *ip=InterfacePolicy::cast(
	    intf->getFirstByType( InterfacePolicy::TYPENAME ) );
	if (ip) {
	    ip->appendRuleAtBottom();
	    showInterfacePolicy( intf );
	}
    }
}

void FirewallDialog::on_del_rule_clicked()
{   
}

