/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FirewallDialog_ipf.cc,v 1.5 2002/04/15 07:38:57 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "fwbuilder/Firewall.hh"
#include "FirewallDialog.hh"
#include "MessageDialog.hh"
#include "fwbuilder/FWObjectDatabase.hh"
#include "Preferences.hh"
#include "ListOfIcons.hh"


using namespace libfwbuilder;


void FirewallDialog::init_ipf()
{
    ((Gtk::Label*)(ipf_log_or_block->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipf_log_body->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(ipf_no_optimisation->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipf_nat_ftp_proxy->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipf_nat_rcmd_proxy->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipf_nat_raudio_proxy->get_child()))->set_alignment(0.0,0.5);

    label557->set_line_wrap(true);


    ipf_log_facility->set_menu(log_facilities_full);
    ipf_log_facility->changed.connect(SigC::slot(this,&FirewallDialog::on_ipf_log_facility_changed));
    ipf_log_facility->show_all();

    ipf_log_level->set_menu(log_levels);
    ipf_log_level->changed.connect(SigC::slot(this,&FirewallDialog::on_ipf_log_level_changed));
    ipf_log_level->show_all();

    label571->set_line_wrap(true);
    label572->set_line_wrap(true);

    ((Gtk::Label*)(ipf_in_out_code->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipf_pass_all_out->get_child()))->set_alignment(0.0,0.5);

    ipf_options.push_back( DialogOption( ipf_in_out_code      , "in_out_code"   ));
    ipf_options.push_back( DialogOption( ipf_pass_all_out     , "pass_all_out"  ));


    ipf_options.push_back( DialogOption( ipf_log_level           , "ipf_log_level"       ));
    ipf_options.push_back( DialogOption( ipf_log_facility        , "ipf_log_facility"    ));
    ipf_options.push_back( DialogOption( ipf_log_or_block        , "ipf_log_or_block"    ));
    ipf_options.push_back( DialogOption( ipf_log_body            , "ipf_log_body"        ));

    ((Gtk::Label*)(ipf_debug->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(ipf_manage_virtual_addr->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(ipf_manage_virtual_addr->get_child()))->set_justify(GTK_JUSTIFY_LEFT);


    ipf_options.push_back( DialogOption( ipf_no_optimisation      , "no_optimisation"      ));
    ipf_options.push_back( DialogOption( ipf_nat_ftp_proxy        , "ipf_nat_ftp_proxy"    ));
    ipf_options.push_back( DialogOption( ipf_nat_rcmd_proxy       , "ipf_nat_rcmd_proxy"   ));
    ipf_options.push_back( DialogOption( ipf_nat_raudio_proxy     , "ipf_nat_raudio_proxy" ));

    ipf_options.push_back( DialogOption( ipf_debug                , "debug"                ));
    ipf_options.push_back( DialogOption( ipf_manage_virtual_addr  , "manage_virtual_addr"  ));
    ipf_options.push_back( DialogOption( ipf_fw_dir               , "firewall_dir"         ));


}

void FirewallDialog::on_ipf_log_facility_changed()
{
    string nl=ipf_log_facility->get_value();
    string ol=options->getStr("ipf_log_facility" );
    if (nl!=ol) on_changed();
}

void FirewallDialog::on_ipf_log_level_changed()
{
    string nl=ipf_log_level->get_value();
    string ol=options->getStr("ipf_log_level" );
    if (nl!=ol) on_changed();
}


