/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: FirewallDialog_iptables.cc,v 1.4 2002/03/18 10:08:40 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include "fwbuilder/Firewall.hh"
#include "FirewallDialog.hh"

using namespace libfwbuilder;

void FirewallDialog::init_iptables()
{
    ((Gtk::Label*)(iptables_firewall_is_in_any->get_child()))->set_alignment( 0.0 , 0.0 );

    ((Gtk::Label*)(iptables_log_tcp_seq->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_log_tcp_opt->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_log_ip_opt->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_numeric_log_levels->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(iptables_debug->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_load_modules->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_no_optimisation->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(iptables_firewall_is_in_any->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_manage_virtual_addr->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_log_all_dropped->get_child()))->set_alignment(0.0,0.5);
    ((Gtk::Label*)(iptables_clamp_MSS_to_MTU->get_child()))->set_alignment(0.0,0.5);

    ((Gtk::Label*)(iptables_accept_new_tcp_with_no_syn->get_child()))->set_alignment(0.0,1.0);
    ((Gtk::Label*)(iptables_accept_new_tcp_with_no_syn->get_child()))->set_justify(GTK_JUSTIFY_LEFT);

    ((Gtk::Label*)(iptables_accept_established->get_child()))->set_alignment(0.0,1.0);
    ((Gtk::Label*)(iptables_accept_established->get_child()))->set_justify(GTK_JUSTIFY_LEFT);


    iptables_log_level->set_menu(log_levels);
    iptables_log_level->changed.connect(SigC::slot(this,&FirewallDialog::on_iptables_log_level_changed));
    iptables_log_level->show_all();

    limit_suffixes.push_back("/second");
    limit_suffixes.push_back("/minute");
    limit_suffixes.push_back("/hour");
    limit_suffixes.push_back("/day");

    iptables_limit_suffix->set_menu(limit_suffixes);
    iptables_limit_suffix->changed.connect( SigC::slot(this,&FirewallDialog::on_iptables_limit_suffix_changed));
    iptables_limit_suffix->show_all();

    actions_on_reject.push_back("ICMP net unreachable");
    actions_on_reject.push_back("ICMP host unreachable");
    actions_on_reject.push_back("ICMP port unreachable");
    actions_on_reject.push_back("ICMP proto unreachable");
    actions_on_reject.push_back("ICMP net prohibited");
    actions_on_reject.push_back("ICMP host prohibited");
    actions_on_reject.push_back("TCP RST");

    iptables_action_on_reject->set_menu(actions_on_reject);
    iptables_action_on_reject->changed.connect(SigC::slot(this,&FirewallDialog::on_iptables_action_on_reject_changed));
    iptables_action_on_reject->show_all();


    iptables_options.push_back( DialogOption( iptables_log_tcp_seq                      , "log_tcp_seq"    ));
    iptables_options.push_back( DialogOption( iptables_log_tcp_opt                      , "log_tcp_opt"    ));
    iptables_options.push_back( DialogOption( iptables_log_ip_opt                       , "log_ip_opt"     ));
					      		                      
    iptables_options.push_back( DialogOption( iptables_log_level                        , "log_level"       ));

    iptables_options.push_back( DialogOption( iptables_numeric_log_levels               , "use_numeric_log_levels" ));


    iptables_options.push_back( DialogOption( iptables_log_prefix                       , "log_prefix"      ));
    iptables_options.push_back( DialogOption( iptables_limit_suffix                     , "limit_suffix"    ));
    iptables_options.push_back( DialogOption( iptables_limit_value                      , "limit_value"     ));
    iptables_options.push_back( DialogOption( iptables_load_modules                     , "load_modules"    ));
					      		                      
    iptables_options.push_back( DialogOption( iptables_debug                            , "debug"           ));
    iptables_options.push_back( DialogOption( iptables_path                             , "script_env_path" ));
					     
    iptables_options.push_back( DialogOption( iptables_action_on_reject                 , "action_on_reject"));
					     
    iptables_options.push_back( DialogOption( iptables_firewall_is_in_any               , "firewall_is_part_of_any_and_networks"));
    iptables_options.push_back( DialogOption( iptables_no_optimisation                  , "no_optimisation"));
    iptables_options.push_back( DialogOption( iptables_manage_virtual_addr              , "manage_virtual_addr"     ));

    iptables_options.push_back( DialogOption( iptables_log_all_dropped                  , "log_all_dropped"));

    iptables_options.push_back( DialogOption( iptables_clamp_MSS_to_MTU                  , "clamp_mss_to_mtu"));

    iptables_options.push_back( DialogOption( iptables_accept_new_tcp_with_no_syn       , "accept_new_tcp_with_no_syn"));
					     
    iptables_options.push_back( DialogOption( iptables_accept_established               , "accept_established"));


}

void FirewallDialog::on_iptables_log_level_changed()
{
    string nl=iptables_log_level->get_value();
    string ol=options->getStr("log_level" );
    if (nl!=ol) on_changed();
}

void FirewallDialog::on_iptables_limit_suffix_changed()
{
    string nl=iptables_limit_suffix->get_value();
    string  ol=options->getStr("limit_suffix" );
    if (nl!=ol) on_changed();
}

void FirewallDialog::on_iptables_action_on_reject_changed()
{
    string naor=iptables_action_on_reject->get_value();
    string oaor=options->getStr("action_on_reject" );
    if (naor!=oaor) on_changed();
}

