/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: InterfaceDialog.cc,v 1.30 2002/04/09 06:25:58 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "fwbuilder/libfwbuilder-config.h"

#include <gtk--/main.h>

#include "fwbuilder/IPAddress.hh"
#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/Group.hh"

#include "InterfaceDialog.hh"
#include "IPAddressWidget.hh"

#include "PixmapButton.hh"

#include "helpers.hh"

using namespace libfwbuilder;

#define GENERAL_PAGE         0
#define SECURITY_LEVEL_PAGE  1
#define NET_ZONE_PAGE        2

InterfaceDialog::InterfaceDialog( Interface *i)
{
    choice=false;
    interface_=i;

    set_position(GTK_WIN_POS_CENTER);

    PixmapButton::addPixmapAndText( *button17 , "Ok"     , "OK"     );
    PixmapButton::addPixmapAndText( *button18 , "Cancel" , "Cancel" );

    button17->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
    button18->set_flags(GTK_CAN_FOCUS|GTK_CAN_DEFAULT);
    button17->grab_default();

    delete_event.connect( SigC::slot(this, &InterfaceDialog::on_delete) );

    button17->show();
    button18->show();


    addr=manage(new IPAddressWidget());
    table1->attach(*addr, 1, 2, 3, 4, GTK_EXPAND|GTK_FILL, 0, 0, 0);

    netmask=manage(new IPAddressWidget());
    table1->attach(*netmask, 1, 2, 4, 5, GTK_EXPAND|GTK_FILL, 0, 0, 0);

    addr->setAssociatedNetmaskWidget( netmask );

    name->grab_focus();

    iface_dialog_notebook->get_nth_page(GENERAL_PAGE)->show();
    iface_dialog_notebook->get_nth_page(SECURITY_LEVEL_PAGE)->hide();
    iface_dialog_notebook->get_nth_page(NET_ZONE_PAGE)->hide();

    show();
}

void InterfaceDialog::useExtCheckbox()
{
    extint->show();
    security_level->hide();
    iface_dialog_notebook->get_nth_page(SECURITY_LEVEL_PAGE)->show();

    sec_level_txt->set_text(
"One interface of the firewall must be marked as 'external'. This interface \
should be connected to the least secure network, usually the Internet"
    );
    sec_level_txt->set_line_wrap(true);
}

void InterfaceDialog::useSecurityLevel()
{
    extint->hide();
    security_level->show();
    iface_dialog_notebook->get_nth_page(SECURITY_LEVEL_PAGE)->show();

    sec_level_txt->set_text(
"Each interface of the firewall must have security level associated with it. \
Security level can be any number between 0 and 100, 0 being least secure \
and 100 being most secure levels. Interface with security level 0 ususally \
serves Internet connection."
    );
    sec_level_txt->set_line_wrap(true);
}

void InterfaceDialog::useNetworkZone()
{
    iface_dialog_notebook->get_nth_page(NET_ZONE_PAGE)->show();

    net_zone_txt->set_text(
"Network zone consists of hosts and networks that can be reached through \
this interface of the firewall. Subnet to which this interface is directly \
attached must be part of its network zone. Other subnets reachable by \
means of routing should alse be added to the network zone.\n\n\
If network zone for this interface consists from only one subnet, you \
can simply choose that network's object in the pull-down \
below. If your network zone should include multiple subnets, \
you need to create an Object Group, then put all hosts and networks \
which are going to be part of the network zone into that group and \
finally choose this group in the pull-down below.\n"
    );
    net_zone_txt->set_line_wrap(true);

    net_zone_combo->get_entry()->set_editable(false);
    net_zone_combo->set_value_in_list(true,true);

    vector<string> obj_names;

    string lbl=" Any ";
    net_zone_objects[lbl]=FWObjectDatabase::db->std.AnyNetworkId;
    obj_names.push_back( lbl );   // empty name

/* TODO: try to make this widget show object with appropriate icon */

    FWObject *groups=
	FWObjectDatabase::db->getById( FWObjectDatabase::db->std.ObjectGroupsId , true );
    if (groups) {
	for (FWObject::iterator i=groups->begin(); i!=groups->end(); i++) {
	    string lbl="Group: "+ (*i)->getName();
	    net_zone_objects[lbl]=(*i)->getId();
	    obj_names.push_back( lbl );
	}
    }

    FWObject *networks=
	FWObjectDatabase::db->getById( FWObjectDatabase::db->std.NetworksId , true );
    if (networks) {
	for (FWObject::iterator i=networks->begin(); i!=networks->end(); i++) {
	    string lbl="Network: "+ (*i)->getName();
	    net_zone_objects[lbl]=(*i)->getId();
	    obj_names.push_back( lbl );
	}
    }
    net_zone_combo->set_popdown_strings( obj_names );
}

bool InterfaceDialog::run()
{
    name->set_text( interface_->getName() );
    label->set_text( interface_->getLabel() );

    if (! interface_->isDyn() ) {
	addr->setAddress( interface_->getStr("address") );
	netmask->setAddress( interface_->getStr("netmask") );
    } else {
	addr->clear();
	addr->set_sensitive(false);
	netmask->clear();
	netmask->set_sensitive(false);
    }

    dyn_addr->set_active( interface_->isDyn() );

    physAddress->set_text( interface_->getPhysicalAddress() );


    if ( iface_dialog_notebook->get_nth_page(SECURITY_LEVEL_PAGE)->is_visible()){
	if (extint->is_visible())
	    extint->set_active( interface_->isExt() );

	if (security_level->is_visible())
	    security_level->set_value( interface_->getSecurityLevel() );
    }

    if ( iface_dialog_notebook->get_nth_page(NET_ZONE_PAGE)->is_visible()) {
	string id=interface_->getStr("network_zone");
	FWObject *nz=FWObjectDatabase::db->getById( id , true );
	if (!id.empty() && nz)  {
/* TODO: this code will change when we switch to a widget showing object name
 * with an icon */
	    string lbl=
		string( (Group::cast(nz)!=NULL)? "Group: ":"Network: ") 
		+ nz->getName();
	    net_zone_combo->get_entry()->set_text( lbl );
	}
//	else  // else use "Any"
//	    net_zone_combo->get_entry()->set_text( obj_names.front() );
    }

    while (true) {
	Gtk::Main::grab_add(*this);
	Gtk::Main::run();
	Gtk::Main::grab_remove(*this);

	if (choice) {
	    if (  ! checkObjectName(name->get_text() ) ) continue;
	    if (  ! dyn_addr->get_active() ) {
		if ( ! checkIPaddress(addr->getAsString()) ) continue;
		if ( ! checkNetmask(netmask->getAsString()) ) continue;
	    }
	}
	break;
    }

    hide();

    if (choice) {
	interface_->setName(name->get_text() );
	interface_->setLabel(label->get_text() );
	interface_->setStr("address",addr->getAsString());
	interface_->setStr("netmask",netmask->getAsString());
	interface_->setBool("dyn",dyn_addr->get_active());
	interface_->setStr("physAddress", physAddress->get_text() );

	if ( iface_dialog_notebook->get_nth_page(SECURITY_LEVEL_PAGE)->is_visible()){
	    if (extint->is_visible())
		interface_->setExt( extint->get_active() );

	    if (security_level->is_visible())
		interface_->setSecurityLevel( security_level->get_value_as_int() );
	}

	if ( iface_dialog_notebook->get_nth_page(NET_ZONE_PAGE)->is_visible()) {
	    string nzname=net_zone_combo->get_entry()->get_text();
	    if ( ! nzname.empty() && !net_zone_objects[nzname].empty() ) {
		interface_->setStr("network_zone", net_zone_objects[nzname] );
	    }
	}
    }
    return(choice);
}


gint InterfaceDialog::on_delete(GdkEventAny* ev)
{
    choice=false;
    Gtk::Main::quit();
    return false;
}

void InterfaceDialog::on_ok_clicked()
{   
    choice=true;
    Gtk::Main::quit();
}

void InterfaceDialog::on_cancel_clicked()
{   
    choice=false;
    Gtk::Main::quit();
}

void InterfaceDialog::on_extint_toggled()
{   
}

void InterfaceDialog::on_dyn_addr_toggled()
{

    if (dyn_addr->get_active()) {
	addr->clear();
	addr->set_sensitive(false);
	netmask->clear();
	netmask->set_sensitive(false);
    } else {
	addr->set_sensitive(true);
	netmask->set_sensitive(true);
    }

}


