/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyDialog.cc,v 1.17 2002/05/09 07:06:01 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/


#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/Firewall.hh"
#include "fwbuilder/FWOptions.hh"
#include "fwbuilder/Policy.hh"
#include "fwbuilder/Management.hh"


#include "PolicyDialog.hh"
#include "helpers.hh"
#include "PolicyList.hh"
#include "main_window.hh"


using namespace libfwbuilder;


PolicyDialog::PolicyDialog(FWObject *obj)
{
    Policy  *pol;
    
    object=obj;
    dialog_buttons=true;

    policylist_ptr=NULL;

    assert( RuleSet::cast(obj)!=NULL );

    pol=Policy::cast(obj);

    policylist_ptr = manage(new PolicyList(pol));

  /*******************************************************************/


    remove();

    Gtk::ScrolledWindow *sw=manage( new Gtk::ScrolledWindow() );
    sw->set_policy(GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    sw->set_name("PolicyListSW");
    add(*sw);
    sw->show();
    
    sw->add_with_viewport(*policylist_ptr);
    policylist_ptr->show();

    set_name("policy_dialog");
     
}

PolicyDialog::~PolicyDialog()
{
//  if (policylist_ptr!=NULL)  delete policylist_ptr;
//  policylist_ptr=NULL;
}


void PolicyDialog::on_policy_line_add()
{
  data_changed_flag(true);
}

void PolicyDialog::on_policy_line_remove()
{
  data_changed_flag(true);
}

void PolicyDialog::wrk2dlg()
{
    if (policylist_ptr!=NULL)  policylist_ptr->build();
}

bool PolicyDialog::dlg2wrk()
{
    return(true);
}

/**
 *  enable and disable menu items which make sence only for this object
 */
void PolicyDialog::updateMainMenu()
{
    Gtk::MenuBar *main_menubar=main_w->getMainMenuBar();
    Gtk::Widget *itm;

    DialogPlugin::updateMainMenu();

    itm=find_widget("policy",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("insert_top",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("append_bottom",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("add_after",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("remove_current",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("move_up",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("move_down",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("rules_druid",main_menubar);
    if (itm) itm->set_sensitive(true);

    itm=find_widget("compile_policy",main_menubar);
    if (itm) itm->set_sensitive(true);

    FWObject *f=object;
    while (Firewall::cast(f)==NULL)
	f=f->getParent();

    Management *mgmt=(Firewall::cast(f))->getManagementObject();
    assert(mgmt!=NULL);
    PolicyInstallScript *pis   = mgmt->getPolicyInstallScript();

    itm=find_widget("install_policy",main_menubar);
    if (itm) itm->set_sensitive( pis->isEnabled() );
}


