/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyList.cc,v 1.35 2002/05/09 07:06:01 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#include <iostream.h>

#include "glademm_support.hh"
#include "helpers.hh"
#include "BuiltinDialog.hh"
#include "gen_popup_menu.hh"
#include "Resources.hh"

#include "fwbuilder/Policy.hh"
#include "fwbuilder/NAT.hh"
#include "fwbuilder/Rule.hh"

#include "PolicyListItem.hh"
#include "PolicyListElement.hh"
#include "PolicyList.hh"

#include "main_window.hh"



using namespace libfwbuilder;

RuleSetList::RuleSetList(RuleSet *p) : Gtk::Table(1,2)
{
    ruleset=p;
    current_selected_row=0;
    current_selected=0;
    rebuild=false;
    width=height=0;

    set_flags( GTK_CAN_FOCUS );

    realize.connect(SigC::slot(this, &RuleSetList::on_realize_event));
    size_allocate.connect(SigC::slot(this, &RuleSetList::on_size_allocate));
    
    bottom=NULL;

    button_release_event.connect_after(
	SigC::slot(this,&RuleSetList::on_button_release_event));

    show();
}

RuleSetList::~RuleSetList()
{
//  Clear();
}

/*
 *   We keep track of our physical size
 */
void RuleSetList::on_size_allocate(GtkAllocation *all)
{
    x=all->x;
    y=all->y;
    width=all->width;
    height=all->height;
}

gint RuleSetList::on_button_release_event(GdkEventButton *ev)
{
    if ( ev->type == GDK_BUTTON_RELEASE && ev->button==3 ) {

	gen_popup_menu         *gpm;

	const char *menu_items0[] = 
	    { "Insert rule at the top",
	      "Append rule at the bottom",
	      NULL,
	      NULL } ;

	if ( ruleset->size()==0 ) {
	    menu_items0[0]="Insert New rule";
	    menu_items0[1]="";
	    menu_items0[2]="Paste Rule";
	    menu_items0[3]=NULL;
	}

	gpm=new gen_popup_menu( menu_items0 );
	gpm->popup(0,0);
	gint menu_choice=gpm->run();
	delete gpm;

	switch (menu_choice) {
	case 0:
	    insertRuleAtTop();
	    break;

	case 1:
	    appendRuleAtBottom();
	    break;

	case 2:
	{
	    FWObject *obj;
	    if ( (obj=FWObjectClipboard::obj_clipboard->getObject())!=NULL &&
		 dynamic_cast<Rule*>(obj)!=NULL ) {
		appendRuleAtBottom( dynamic_cast<Rule*>(obj) );
	    }
	}
	break;
	}
    }
    return( true );
}


void RuleSetList::move_focus(GtkDirectionType dir)
{
    gint r,c;

    if (current_selected!=0) {

//	current_selected->deselect();

	r=current_selected->get_row();
	c=current_selected->get_col();

	switch (dir ) {
	case GTK_DIR_UP:     r-=1;    break;
	case GTK_DIR_DOWN:   r+=1;    break;
	case GTK_DIR_LEFT:   c-=1;    break;
	case GTK_DIR_RIGHT:  c+=1;    break;
	default:         return;
	}
	select_element(r,c);
    } else
	select_element(0,0);
}

void RuleSetList::select_element(gint r,gint c)
{
    bool going_up=false;

    PolicyListElement *pe;
    if ( (pe=get_element(r,c))!=0 ) {

	if ( current_selected!=0) {
	    int cr=current_selected->get_row();
	    current_selected->deselect();
	    current_selected=NULL;
	    if (cr!=r) {
		deactivate_row(cr);
		activate_row(r);
		going_up= (r<cr);
	    }
	}

	set_current_selected(pe);
	pe->select(false);
	if (going_up) pe->select_last_child();
	else          pe->select_first_child();
    }
}

void RuleSetList::deselect_current()
{
    if ( current_selected!=0) {
	current_selected->deselect();
	deactivate_row(current_selected->get_row());
	current_selected=NULL;
    }
}

void RuleSetList::set_current_selected(PolicyListElement *pe)
{
    current_selected=pe;
    current_selected_row=pe->get_row();
}

void RuleSetList::request_focus()
{
    if (!has_focus()) {
	grab_focus();
	draw_focus();
    }
}


void RuleSetList::activate_row(gint row)
{
    PolicyListElement *pe;

    deactivate_row( current_selected_row );
    current_selected_row=row;

    if ( ! table_elements.empty() ) {
	map<int,Gtk::Widget*>::iterator i;
	for (i=table_elements[row].begin(); i!=table_elements[row].end(); ++i) {
	    if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		pe=(PolicyListElement*)( (*i).second );
		pe->activate();
	    }
	}
    }
}

void RuleSetList::activate_rule(gint r)
{
    activate_row( get_row_by_rule_num(r) );
}

void RuleSetList::deactivate_row(gint row)
{
//    deselect_current();

    PolicyListElement *pe;
    if ( ! table_elements.empty() ) {
	map<int,Gtk::Widget*>::iterator i;
	for (i=table_elements[row].begin(); i!=table_elements[row].end(); ++i) {
	    if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		pe=(PolicyListElement*)( (*i).second );
		pe->deactivate();
	    }
	}
    }
}

void RuleSetList::deactivate_rule(gint r)
{
    deactivate_row( get_row_by_rule_num(r) );
}


void RuleSetList::schedule_rebuild(int r)
{
    activate_r=r;
    Gtk::Main::idle.connect(slot(this,&RuleSetList::rebuild_when_idle));
}

gint RuleSetList::rebuild_when_idle()
{
    build();
    activate_rule(activate_r);
    return false;
}


void RuleSetList::on_realize_event()
{
}


void RuleSetList::set_data_changed_flag(bool flag)
{
    BuiltinDialog *b=(BuiltinDialog*)glademm_get_Widget("BuiltinDialog");
    assert (b!=NULL);
    b->data_changed_flag(flag);
}


void RuleSetList::Clear()
{
    if ( ! table_elements.empty() ) {
	map<int, map<int,Gtk::Widget*> >::iterator k;
	for (k=table_elements.begin(); k!=table_elements.end(); ++k) {
/*  TODO: is there extra assignment here? */ 
	    map<int,Gtk::Widget*> row_list= (*k).second;
	    map<int,Gtk::Widget*>::iterator i;
	    for (i=row_list.begin(); i!=row_list.end(); ++i) {
		Gtk::Widget *w=( (*i).second );
		assert( w!=NULL );
		w->hide();
		remove(*w);
	    }
	}
    }
    table_elements.clear();
    current_selected=0;
}

Gtk::Widget* RuleSetList::_register_child(int row,int col,Gtk::Widget *c)
{
    table_elements[row][col]=c;

    return c;
}


void RuleSetList::_add_hseparator(gint row,gint col)
{
    Gtk::HSeparator *sep= manage(new Gtk::HSeparator());
    attach( *sep , 0 , col , row , row+1 , GTK_FILL, GTK_FILL, 0, 0);
    _register_child(row,col,sep);
}

void RuleSetList::_add_vseparator(gint row,gint col)
{
    Gtk::VSeparator *sep= manage(new Gtk::VSeparator());
    attach( *sep , col , col+1 , row , row+1 , 0, GTK_FILL, 0, 0);
    _register_child(row,col,sep);
}


gint RuleSetList::get_row_by_rule_num(gint rule_n)
{
    return( rule_n+1 );     // if there are no separators
}

gint RuleSetList::get_rule_by_row(gint row_n)
{
    return( row_n-1 );       // if there are no separators
}

int RuleSetList::resizeTable()
{
//    int tbl_size=ruleset->size()*2+1 +1;
//    resize( tbl_size , countElements()*2 );

    int tbl_size=ruleset->size() +1;
    resize( tbl_size , countElements() );
    return tbl_size;
}

void RuleSetList::removeWidgetsInRow(int row)
{
    if ( ! table_elements.empty() ) {
	map<int,Gtk::Widget*>::iterator i;
	for (i=table_elements[row].begin(); i!=table_elements[row].end(); ++i) {
	    Gtk::Widget *w= (*i).second;
	    remove(*w);
	}	
	table_elements[row].clear();
    }
}

void RuleSetList::swapWidgetsInRows(int row1, int row2)
{
    if ( ! table_elements.empty() ) {

	map<int,Gtk::Widget*>  rm1=table_elements[row1];
	map<int,Gtk::Widget*>  rm2=table_elements[row2];
	table_elements[row1].clear();
	table_elements[row2].clear();

	map<int,Gtk::Widget*>::iterator i;
	for (i=rm1.begin(); i!=rm1.end(); ++i) {
	    if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		PolicyListElement* pe=dynamic_cast<PolicyListElement*>( (*i).second );
		int c=pe->get_col();

		pe->ref();
		remove(*pe);
		pe->set_row(row2);
		attachTableElement(row2,c,pe);
		pe->unref();
	    }
	}	
	for (i=rm2.begin(); i!=rm2.end(); ++i) {
	    if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		PolicyListElement* pe=dynamic_cast<PolicyListElement*>( (*i).second );
		int c=pe->get_col();

		pe->ref();
		remove(*pe);
		pe->set_row(row1);
		attachTableElement(row1,c,pe);
		pe->unref();
	    }
	}	

    }

}

void RuleSetList::moveWidgetsDown(int first,int last)
{
    PolicyListElement *pe;
    int last_row = (last!=-1) ? last : get_row_by_rule_num( ruleset->size() +1 );

    if ( ! table_elements.empty() ) {

	for (int r=last_row-1; r>=first; --r) {
	    map<int,Gtk::Widget*>::iterator i;
	    for (i=table_elements[r].begin(); i!=table_elements[r].end(); ++i) {
		if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		    pe=dynamic_cast<PolicyListElement*>( (*i).second );
		    int c=pe->get_col();

		    pe->ref();
		    remove(*pe);

		    pe->set_row(r+1);
		    attachTableElement(r+1,c,pe);
		    pe->unref();
		}
	    }
	}
    }
}

void RuleSetList::moveWidgetsUp(int first,int last)
{
    PolicyListElement *pe;
    int last_row = (last!=-1) ? last : get_row_by_rule_num( ruleset->size() +1 );

    if ( ! table_elements.empty() ) {

	for (int r=first; r<=last_row; ++r) {
	    map<int,Gtk::Widget*>::iterator i;
	    for (i=table_elements[r].begin(); i!=table_elements[r].end(); ++i) {
		if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		    pe=dynamic_cast<PolicyListElement*>( (*i).second );
		    int c=pe->get_col();

		    pe->ref();
		    remove(*pe);

		    pe->set_row(r-1);
		    attachTableElement(r-1,c,pe);
		    pe->unref();
		}
	    }
	    table_elements[r].clear();
	}
    }
}

void RuleSetList::copyRuleContent(Rule *dst, Rule *src)
{
    string id=dst->getId();
    int     p=dst->getPosition();

    if ( src->isDisabled() ) dst->disable();
    else                     dst->enable();

    map<string, string>::const_iterator i;
    for(i=dst->dataBegin(); i!=dst->dataEnd(); ++i) {
	string f= (*i).first;
	dst->setStr(f, src->getStr(f) );
    }

    dst->setComment( src->getComment() );

    list<FWObject*>::iterator j;
    for(j=dst->begin(); j!=dst->end(); ++j) {
	string    dtype= (*j)->getTypeName();
	FWObject *selem= src->getFirstByType(dtype);
	if (selem!=NULL) 
	    (*j)->duplicate(selem);
    }

    if (id!="")	dst->setId(id);
    dst->setPosition(p);
}


void RuleSetList::insertRuleAtTop(Rule *r)
{
    insertRuleBefore(0,r);
    buildTableBottom( get_row_by_rule_num( ruleset->getRuleSetSize() ) );
}

void RuleSetList::insertRuleBefore(gint rule_n,Rule *r)
{
    if (r!=NULL && 
	ruleset->getTypeName()==Policy::TYPENAME && 
	r->getTypeName()!=PolicyRule::TYPENAME)  return;
    if (r!=NULL && 
	ruleset->getTypeName()==NAT::TYPENAME    && 
	r->getTypeName()!=NATRule::TYPENAME   )  return;

    Rule *newrule;
    int   nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( (newrule=ruleset->insertRuleBefore(nr))!=NULL ) {
	if (r!=NULL) {
	    copyRuleContent(newrule,r);
	}
	resizeTable();

	moveWidgetsDown( get_row_by_rule_num(nr) );
	buildRule( nr , newrule );
	renumberRules();
	activate_rule( nr );
	
	set_data_changed_flag(true);
    }
    buildTableBottom( get_row_by_rule_num( ruleset->getRuleSetSize() ) );
}

void RuleSetList::appendRuleAfter(gint rule_n,Rule *r)
{
    if (r!=NULL && 
	ruleset->getTypeName()==Policy::TYPENAME && 
	r->getTypeName()!=PolicyRule::TYPENAME)  return;
    if (r!=NULL && 
	ruleset->getTypeName()==NAT::TYPENAME    && 
	r->getTypeName()!=NATRule::TYPENAME   )  return;

    Rule *newrule;
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( (newrule=ruleset->appendRuleAfter( nr ))!=NULL ) {
	if (r!=NULL) {
	    copyRuleContent(newrule,r);
	}
	resizeTable();

	moveWidgetsDown( get_row_by_rule_num(nr+1) );
	buildRule( nr+1 , newrule );
	renumberRules();
	activate_rule( nr );
	
	set_data_changed_flag(true);
    }
    buildTableBottom( get_row_by_rule_num( ruleset->getRuleSetSize() ) );
}

void RuleSetList::appendRuleAtBottom(Rule *r)
{
    appendRuleAfter( ruleset->getRuleSetSize()-1 , r );
    buildTableBottom( get_row_by_rule_num( ruleset->getRuleSetSize() ) );
}

void RuleSetList::delRule(gint rule_n)
{
    deselect_current();

    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->deleteRule(nr) ) {

	deactivate_row(current_selected_row);
	current_selected_row=0;
	removeWidgetsInRow( get_row_by_rule_num(nr) );
	moveWidgetsUp( get_row_by_rule_num(nr+1) );
	renumberRules();
	activate_rule( nr );

	set_data_changed_flag(true);
    }
}

void RuleSetList::moveRuleUp(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->moveRuleUp(nr) ) {
	swapWidgetsInRows( get_row_by_rule_num(nr-1),get_row_by_rule_num(nr) );
	renumberRules();
	activate_rule(nr-1);
	set_data_changed_flag(true);
    }
}

void RuleSetList::moveRuleDown(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->moveRuleDown(nr) ) {
	swapWidgetsInRows( get_row_by_rule_num(nr),get_row_by_rule_num(nr+1) );
	renumberRules();
	activate_rule(nr+1);
	set_data_changed_flag(true);
    }
}

void RuleSetList::moveRuleTo(int rule1,int rule2)
{
    map<int,Gtk::Widget*>::iterator i;
    PolicyListElement*  pe;

    if ( rule1==rule2 || rule1+1==rule2) return;

    if ( ruleset->moveRule(rule1,rule2) ) {
	int row1=get_row_by_rule_num(rule1);
	int row2=get_row_by_rule_num(rule2);

	map<int,Gtk::Widget*> rm1=table_elements[row1];

	int dest_row;

	for (i=rm1.begin(); i!=rm1.end(); ++i) {
	    if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		pe=dynamic_cast<PolicyListElement*>( (*i).second );
		pe->ref();
		remove(*pe);
	    }
	}
	table_elements[row1].clear();


	if (row1<row2)   {
	    moveWidgetsUp( row1+1,row2-1 );
	    dest_row=row2-1;
	} else {
	    moveWidgetsDown( row2,row1 );
	    dest_row=row2;
	}

	for (i=rm1.begin(); i!=rm1.end(); ++i) {
	    if (dynamic_cast<PolicyListElement*>( (*i).second )!=NULL) {
		pe=dynamic_cast<PolicyListElement*>( (*i).second );
		int c=pe->get_col();

		pe->set_row(dest_row);
		attachTableElement(dest_row,c,pe);
		pe->unref();
	    }
	}

	renumberRules();
	activate_rule( get_rule_by_row(dest_row) );
    }
	
    set_data_changed_flag(true);
}

void RuleSetList::disableRule(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->disableRule(nr) ) {
	int r=get_row_by_rule_num( nr );
	map<int,Gtk::Widget*>::iterator i;
	for (i=table_elements[r].begin(); i!=table_elements[r].end(); ++i) {
	    if (dynamic_cast<PolicyListRuleNum*>( (*i).second )!=NULL) {
		PolicyListRuleNum* pen=dynamic_cast<PolicyListRuleNum*>( (*i).second );
		pen->setNeg(true);
		break;
	    }
	}

	set_data_changed_flag(true);
    }
}

void RuleSetList::enableRule(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    if ( ruleset->enableRule(nr) ) {
	int r=get_row_by_rule_num( nr );
	map<int,Gtk::Widget*>::iterator i;
	for (i=table_elements[r].begin(); i!=table_elements[r].end(); ++i) {
	    if (dynamic_cast<PolicyListRuleNum*>( (*i).second )!=NULL) {
		PolicyListRuleNum* pen=dynamic_cast<PolicyListRuleNum*>( (*i).second );
		pen->setNeg(false);
		break;
	    }
	}

	set_data_changed_flag(true);
    }
}

bool  RuleSetList::isRuleDisabled(gint rule_n)
{
    int nr;
    nr=(rule_n==-1)?get_rule_by_row(current_selected_row):rule_n;
    return ( ruleset->isRuleDisabled(nr) );
}

PolicyListElement* RuleSetList::get_element(gint r,gint c)
{
    if ( ! table_elements.empty() )
	return dynamic_cast<PolicyListElement*>( table_elements[r][c] );

    return(NULL);
}


void RuleSetList::attachTableElement(int row, int col, Gtk::Widget *pe,guint x_attach,guint y_attach)
{
    pe->set_user_data(this);
    attach( *pe , col , col+1 , row , row+1 , x_attach, y_attach, 0, 0);
    if (dynamic_cast<PolicyListElement*>(pe)!=NULL)
	dynamic_cast<PolicyListElement*>(pe)->deactivate();
    pe->show();

    _register_child(row,col,pe);
}

void RuleSetList::createTitleButton( int row, int col,
				     const string &ttl,
				     guint x_attach,guint y_attach)
{
    const char  *s   = Resources::global_res->getRuleElementResourceStr( ttl.c_str() , "label" );
    Gtk::Button *btn = manage(new Gtk::Button( (s!=NULL)?s:ttl ));
    attachTableElement(row,col,btn,x_attach,y_attach);
}

int  RuleSetList::countElements() { return 0; }

void RuleSetList::renumberRules()
{
    PolicyListRuleNum *pen;

    if ( ! table_elements.empty() ) {

	for (unsigned r=1,rule_n=0; r<table_elements.size(); ++r,++rule_n) {
	    map<int,Gtk::Widget*>::iterator i;
	    for (i=table_elements[r].begin(); i!=table_elements[r].end(); ++i) {
		if (dynamic_cast<PolicyListRuleNum*>( (*i).second )!=NULL) {
		    pen=dynamic_cast<PolicyListRuleNum*>( (*i).second );
		    pen->setNum(rule_n);
		    break;
		}
	    }
	}
    }
}

void RuleSetList::buildTitles(){}

void RuleSetList::build()
{
    int   rule_n;

    hide();

    Clear();

    resizeTable();

    buildTitles();

    rule_n=0;

    list<FWObject*>::iterator m;
    for (m=ruleset->begin(); m!=ruleset->end(); ++m,++rule_n ) {
	buildRule( rule_n , *m );
    }

    buildTableBottom( get_row_by_rule_num(rule_n)+1 );

    show_all();
}


void RuleSetList::buildRule(int rule_no, libfwbuilder::FWObject* rule) {}

void RuleSetList::buildTableBottom(int row)
{
    if (bottom!=NULL) remove(*bottom);

    bottom = manage(new Gtk::EventBox());

    attach( *bottom , 0 , countElements()*2 , row , row+1 , 
	    GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 1);

    bottom->show();

    activate_rule(0);
    select_element(0,0);
    
    request_focus();
}

/***************************************************************************/

void PolicyList::buildTitles()
{
    gint                row, col;

    row=0;
    col=0;

    createTitleButton(row,col,"Num",GTK_FILL,0);            col++;
    createTitleButton(row,col,"Src",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"Dst",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"Srv",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"Act",GTK_FILL,0);            col++;
    createTitleButton(row,col,"When",GTK_FILL,0);           col++;
    createTitleButton(row,col,"Opt",GTK_FILL,0);            col++;
    createTitleButton(row,col,"Com",GTK_FILL|GTK_EXPAND,0); col++;
}

int  PolicyList::countElements() { return 8; }

void PolicyList::buildRule(int rule_n, FWObject* r)
{
    int row=get_row_by_rule_num( rule_n );
    int col;
    PolicyListElement  *pe;

    if (PolicyRule::cast(r)!=NULL) {
	PolicyRule *rule=PolicyRule::cast(r);

	col=0;

	pe = manage( new PolicyListRuleNum(row,col,rule_n,rule->isDisabled()));
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getSrc()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getDst()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getSrv()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleAction(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getWhen()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleOpt(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleComment(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;


	row++;
    }


}



/***************************************************************************/

void InterfacePolicyList::buildTitles()
{
    gint                row, col;

    row=0;
    col=0;

    createTitleButton(row,col,"Num",GTK_FILL,0);            col++;
    createTitleButton(row,col,"Src",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"Dst",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"Srv",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"Act",GTK_FILL,0);            col++;
    createTitleButton(row,col,"Dir",GTK_FILL,0);            col++;
    createTitleButton(row,col,"Opt",GTK_FILL,0);            col++;
    createTitleButton(row,col,"Com",GTK_FILL|GTK_EXPAND,0); col++;
}

int  InterfacePolicyList::countElements() { return 8; }

void InterfacePolicyList::buildRule(int rule_n, FWObject* r)
{
    int row=get_row_by_rule_num(rule_n);
    int col;
    PolicyListElement  *pe;

    if (PolicyRule::cast(r)!=NULL) {
	PolicyRule *rule=PolicyRule::cast(r);

	col=0;

	pe=manage(new PolicyListRuleNum(row,col,rule_n,rule->isDisabled()));
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getSrc()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getDst()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getSrv()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleAction(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleDir(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleOpt(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleComment(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;


	row++;
    }
}

/***************************************************************************/

void NATList::buildTitles()
{
    gint                row, col;

    row=0;
    col=0;

    createTitleButton(row,col,"Num",GTK_FILL,0);            col++;

    createTitleButton(row,col,"OSrc",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"ODst",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"OSrv",GTK_FILL|GTK_EXPAND,0); col++;

    createTitleButton(row,col,"TSrc",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"TDst",GTK_FILL|GTK_EXPAND,0); col++;
    createTitleButton(row,col,"TSrv",GTK_FILL|GTK_EXPAND,0); col++;

    createTitleButton(row,col,"Com",GTK_FILL|GTK_EXPAND,0); col++;
}

int  NATList::countElements() { return 8; }

void NATList::buildRule(int rule_n, FWObject* r)
{
    int row=get_row_by_rule_num(rule_n);
    int col;
    PolicyListElement  *pe;

    if (NATRule::cast(r)!=NULL) {
	NATRule *rule=NATRule::cast(r);

	col=0;

	pe=manage(new PolicyListRuleNum(row,col,rule_n,rule->isDisabled()));
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getOSrc()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getODst()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getOSrv()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getTSrc()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getTDst()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListElement(row,col,rule->getTSrv()) );
	attachTableElement(row,col,pe);
	col++;

	pe = manage( new PolicyListRuleComment(row,col,rule) );
	attachTableElement(row,col,pe);
	col++;

	row++;
    }
}

