/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: TCPServiceDialog.cc,v 1.15 2002/05/09 07:06:02 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "fwbuilder/libfwbuilder-config.h"

#include "TCPServiceDialog.hh"
#include "fwbuilder/TCPService.hh"
#include "MessageDialog.hh"
#include "helpers.hh"

using namespace libfwbuilder;

TCPServiceDialog::TCPServiceDialog(FWObject *obj) 
{ 
    object=obj;
}

void TCPServiceDialog::wrk2dlg()
{
    TCPService *tcp=TCPService::cast(object);

    name->set_text(object->getName());
    dst_range_start->set_value( object->getInt("dst_range_start"));
    dst_range_end->set_value( object->getInt("dst_range_end"));
    src_range_start->set_value( object->getInt("src_range_start"));
    src_range_end->set_value( object->getInt("src_range_end"));
    comment->set_point(0);
    comment->forward_delete( comment->get_length() );
    comment->insert(object->getComment());

    syn_flag->set_active( tcp->getSYNFlag() );
    ack_flag->set_active( tcp->getACKFlag() );
    fin_flag->set_active( tcp->getFINFlag() );
    rst_flag->set_active( tcp->getRSTFlag() );
    psh_flag->set_active( tcp->getPSHFlag() );
    urg_flag->set_active( tcp->getURGFlag() );

    name->grab_focus();
}

bool TCPServiceDialog::dlg2wrk()
{
    TCPService *tcp=TCPService::cast(object);

/*
 *  check if name is valid
 */
    if ( ! checkObjectName(name->get_text()) ) return(false);

    object->setName( name->get_text() );
    object->setComment( comment->get_chars(0,comment->get_length()) );

    object->setInt( "dst_range_start" , 
		      dst_range_start->get_value_as_int() );
    object->setInt( "dst_range_end" , 
		      dst_range_end->get_value_as_int() );

    if (object->getInt("dst_range_end")==0)
	object->setInt( "dst_range_end" , 
			  object->getInt("dst_range_start"));

    object->setInt( "src_range_start" ,
		      src_range_start->get_value_as_int() );
    object->setInt( "src_range_end" ,
		      src_range_end->get_value_as_int() );

    tcp->setSYNFlag( syn_flag->get_active() );
    tcp->setACKFlag( ack_flag->get_active() );
    tcp->setFINFlag( fin_flag->get_active() );
    tcp->setRSTFlag( rst_flag->get_active() );
    tcp->setPSHFlag( psh_flag->get_active() );
    tcp->setURGFlag( urg_flag->get_active() );

    return(true);
}


void TCPServiceDialog::on_syn_flag_toggled()
{   
  data_changed_flag(true);
}

void TCPServiceDialog::on_ack_flag_toggled()
{   
  data_changed_flag(true);
}

void TCPServiceDialog::on_rst_flag_toggled()
{   
  data_changed_flag(true);
}

void TCPServiceDialog::on_fin_flag_toggled()
{   
  data_changed_flag(true);
}

void TCPServiceDialog::on_changed()
{
  data_changed_flag(true);
}
