/* 

                          Firewall Builder

                 Copyright (C) 2000 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: fwbuilder.cc,v 1.79 2002/05/11 17:35:01 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

#ifdef HAVE_LOCALE_H
#include <locale.h>
#endif

#include <unistd.h>

#include <gtk--/main.h>
#include <gdk-pixbuf/gdk-pixbuf.h>

#include <glib.h>

#include "fwbuilder/FWObject.hh"
#include "FWObjectDatabaseGUI.hh"
#include "helpers.hh"

#include "main_window.hh"
#include "FWObjectBook.hh"

#include "Resources.hh"
#include "Preferences.hh"
#include "MessageDialog.hh"
#include "fwbuilder/Tools.hh"
#include "fwbuilder/XMLTools.hh"

#include "glademm_support.hh"

#include "fwbd/fwbd.h"

#include <iostream.h>

#ifdef __MINGW32__
  #include <getopt.h>
  #include <winsock2.h>
#else
  #ifdef HAVE_GETOPT_H
    #include <getopt.h>
  #else
    #include <stdlib.h>
  #endif
#endif

class StaticInitializer
{
    public:
    
    StaticInitializer()
    {
        /* init threads */
        g_thread_init(NULL);
        
        libfwbuilder::init();
    }
} init;

using namespace libfwbuilder;

static char      filename[128];
main_window      *main_w     = NULL;


#ifdef __MINGW32__
# define __FWB_ENTER	 gdk_threads_enter()
# define __FWB_LEAVE     WSACleanup(); \
 			 gdk_threads_leave(); \
			 __freeCommandLine(argc,argv)
#else
# define __FWB_ENTER
# define __FWB_LEAVE	gdk_threads_leave()
#endif

/**
 *   This handler gets called whenever gtk event loop is stopped via 
 *   Gtk::Main::quit()
 */

gint quit_handler()
{
    if (gtk_main_level()==1) {
// this is the  end of program


    }
    return true;
}

#ifdef __MINGW32__
void __parseCommandLine(string cmdline,int& argc, char**& argv)
{
  int pos = 0;
  argc = 1;
  argv = (char**)malloc(argc);
  char execFname[255];
  int sz = GetModuleFileName(GetModuleHandle(NULL),execFname,255);
  argv[0] = (char*)malloc(sz+1);
  strcpy(argv[0],execFname);

  while(pos != -1)
  {
       int p = cmdline.find_first_of(" ",pos);
       if (p == -1 && pos != cmdline.length() - 1)
           p = cmdline.length();
       if (p != -1)
       {
          argv = (char**)realloc(argv,++argc);
          char s[255];
	  int n = cmdline.copy(s,p - pos,pos);
          argv[argc-1] = (char*)malloc(n+1);
          strncpy(argv[argc-1],s,n);
          argv[argc-1][n+1] = '\0';
       }
       pos = cmdline.find_first_not_of(" ",p);
  }
}

void __freeCommandLine(int argc,char** argv)
{
  for (int i = 0; i < argc; i++)
     free(argv[argc]);
  free(argv);
}

void usage()
{
    MessageBox(NULL,"Usage: fwbuilder [-?hv] [-f filename]","Command line error", MB_OK);
}
#else

void usage()
{
    cerr << "Usage: fwbuilder [-?hv] [-f filename]\n";
}
#endif



#ifdef __MINGW32__

int WINAPI WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,
		   LPSTR lpCmdLine,int nCmdShow )
#else

int main(int argc, char * const *argv)

#endif
 
{   
#ifdef HAVE_SETLOCALE
    setlocale(LC_ALL, "");
#endif

    /* Init XML parser */
    XMLTools::initXMLTools();

    /* open-ssl related initializations */
    fwbd_init();

#ifdef __MINGW32__
     WORD wVersionRequested;
     WSADATA wsaData;

     wVersionRequested = MAKEWORD( 2, 2 );
     WSAStartup( wVersionRequested, &wsaData );

     int argc = 0;
     char** argv = NULL;
     __FWB_Instance = hInstance;
    __parseCommandLine(lpCmdLine,argc,argv);
#else
     gdk_threads_enter();
#endif

    Gtk::Main m(&argc, (char***)(&argv) );

#ifdef __MINGW32__
    gtk_rc_parse( TEMPLATE_DIR "/gtkrc.win" );
#else
    gtk_rc_parse( TEMPLATE_DIR "/gtkrc" );
#endif


    int c;
    while ((c = getopt (argc , argv , "?hvf:")) != EOF )
	switch (c) {
	case '?':
	case 'h':
	    usage();
	    exit(0);

	case 'f':
	    strcpy(filename, optarg);
	    break;

	case 'v':
	    cout << VERSION << endl;
	    exit(0);
	}
    

/* pretty much everything else in the GUI uses resources, so we start here */
    try  
    {
        new Resources();       // creates singleton Resources::global_res
    } catch(FWException &ex)  
    {
        /*
         *  Error loading resources. MessageDialog needs Resources (for
         *  icons), so we can't use it to show error message.  
         */
	cerr << ex.toString() << endl;
	__FWB_LEAVE;
        exit(1);
    }


    try  {
        new Preferences();     // creates singleton Preferences::global_prefs
    } catch(FWException &ex)  {
 	__FWB_ENTER;
	MessageDialog::Error(ex.toString());
 	__FWB_LEAVE;
        exit(1);
    }

   
    main_w = new main_window();

    if ( Preferences::global_prefs->getOptBool(
	     "/FWBuilderPreferences/UI/RememberWindowPositionAndSize") ) {

	main_w->restoreWindowConfiguration();
    } 

    main_w->restoreWindowPosition();

    main_w->show_all();

#ifdef __MINGW32__
    __fwb_win32_loadIcon("FWBuilderWin32Ico",main_w);
#endif


    new FWObjectClipboard(); // creates singleton 
    new FWObjectDatabaseGUI();  // creates db and scratchpad

    try   {
	if(strlen(filename)!=0)  {
	    //load file given on command line
	    FWObjectDatabaseGUI::getInstance()->load(filename);
	} else  {
	    //load default file
	    string dbTemplateFile = 
		TEMPLATE_DIR  "/" + FWObjectDatabaseGUI::DB_TEMPLATE_FILE_NAME;
	    FWObjectDatabaseGUI::getInstance()->load(dbTemplateFile);
	    FWObjectDatabaseGUI::getInstance()->setFileName("");
	}
    } catch(FWException &ex)  {
 	__FWB_ENTER;
	MessageDialog::LongTextError(ex.toString());
 	__FWB_LEAVE;
	exit(1);
    }
   
    FWObjectBook *book=(FWObjectBook*)glademm_get_Widget("FWObjectBook");
    assert (book!=NULL);

    book->build();
    main_w->setCurrentLibrary("");
    main_w->schedule_open_object(FWObjectDatabaseGUI::db->std.rootId);

    Gtk::Main::quit.connect( SigC::slot( &quit_handler )  ) ;

   __FWB_ENTER;
    m.run();
    gdk_threads_leave();

    if ( Preferences::global_prefs->getOptBool(
	     "/FWBuilderPreferences/UI/RememberWindowPositionAndSize") ) {

	main_w->saveWindowConfiguration();
	
    }

    delete main_w;
    delete Preferences::global_prefs;
    delete Resources::global_res;

    FWObjectDatabaseGUI::ScratchPad->destroyChildren();
    FWObjectDatabaseGUI::db->destroyChildren();
    delete FWObjectDatabaseGUI::ScratchPad;
    delete FWObjectDatabaseGUI::db;

    delete FWObjectClipboard::obj_clipboard;

   __FWB_LEAVE;
    return 0;
}

