/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_ipt_writers.cc,v 1.8 2002/05/15 16:49:19 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "NATCompiler_ipt.hh"

#include "fwbuilder/AddressRange.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/CustomService.hh"
#include "fwbuilder/Policy.hh"

#include <iostream.h>
#include <streambuf.h>
#include <iomanip.h>
#include <fstream>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;



/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */

void NATCompiler_ipt::_printChainDirectionAndInterface(NATRule *rule)
{
    string       iface_id = rule->getStr("interface_id");
    Interface *rule_iface = fw_interfaces[iface_id];
    string iface_name= (rule_iface!=NULL) ? rule_iface->getName() : "";

    output << "-A " << rule->getStr("ipt_chain") << " ";

    switch (rule->getRuleType()) {
    case NATRule::SNAT:  
	if (!iface_name.empty()) output << "-o " << iface_name;
	break;
    case NATRule::Masq:  
	if (!iface_name.empty()) output << "-o " << iface_name;
	break;
    case NATRule::DNAT:  
	if (!iface_name.empty()) output << "-i " << iface_name;
	break;
    case NATRule::Redirect: 
	if (!iface_name.empty()) output << "-i " << iface_name;
	break;
    }
    output << " ";
}



void NATCompiler_ipt::_printProtocol(Service *srv)
{
    if ( ! srv->isAny() && ! CustomService::isA(srv) ) {
        output << "-p " <<  srv->getProtocolName() << " ";
    }
}

void NATCompiler_ipt::_printMultiport(NATRule *rule)
{
    if( rule->getBool("ipt_multiport"))
	output << "-m multiport ";
}


string NATCompiler_ipt::_printPorts(int rs,int re)
{
    std::ostringstream  str;

    if (rs<0) rs==0;
    if (re<0) re==0;

    if (rs>0 || re>0) {
	if (rs==re)  str << rs;
	else
	    if (rs==0 && re!=0)      str << "-" << re;
	    else
		if (rs!=0 && re==65535)  str << rs << "-";
		else {
		    str << rs << "-" << re;
		}
    }
    return str.str();
}

string NATCompiler_ipt::_printICMP(ICMPService *srv)
{
    std::ostringstream  str;
    if (ICMPService::isA(srv) && srv->getInt("type")!=-1) {
	str << srv->getStr("type");
	if (srv->getInt("code")!=-1) 
	    str << "/" << srv->getStr("code") << " ";
    }
    return str.str();
}

string NATCompiler_ipt::_printIP(IPService *srv)
{
    std::ostringstream  str;
    if (IPService::isA(srv) ) {
	if (srv->getBool("fragm") || srv->getBool("short_fragm"))
	    str << "-f ";

        if  (srv->getBool("lsrr") ||
             srv->getBool("ssrr") ||
             srv->getBool("rr") ||
             srv->getBool("ts") ) str << " -m ipv4options ";

        if  (srv->getBool("lsrr")) str << " --lsrr";
        if  (srv->getBool("ssrr")) str << " --ssrr";
        if  (srv->getBool("rr"))   str << " --rr";
        if  (srv->getBool("ts"))   str << " --ts";
    }
    return str.str();
}

string NATCompiler_ipt::_printSrcPorts(Service *srv)
{
    std::ostringstream  str;
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int rs=srv->getInt("src_range_start");
	int re=srv->getInt("src_range_end");
	str << _printPorts(rs,re);
    }
    return str.str();
}

string NATCompiler_ipt::_printDstPorts(Service *srv)
{
    std::ostringstream  str;
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int rs=srv->getInt("dst_range_start");
	int re=srv->getInt("dst_range_end");
	str << _printPorts(rs,re);
    }
    return str.str();
}

/*
 * we made sure that all services in rel  represent the same protocol
 */
void NATCompiler_ipt::_printSrcService(RuleElementOSrv  *rel)
{
    Service *srv= rel->getFirst(true);

    if (rel->size()==1) {
	if (UDPService::isA(srv) || TCPService::isA(srv)) {
	    string str=_printSrcPorts( srv );
	    if (! str.empty() ) output << "--source-port " << str << " ";
	}
    } else {
/* use multiport */

	string str;
	bool  first=true;
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];

	    Service *s=Service::cast( o );
	    assert(s);
	    if (UDPService::isA(srv) || TCPService::isA(srv)) {
		if (!first) str+=",";
		str+= _printSrcPorts( s );
		if (!str.empty()) first=false;
	    }
	}
	if ( !str.empty() ) {
	    output << "--source-port ";
	    output << str << " ";
	}
    }
}

void NATCompiler_ipt::_printDstService(RuleElementOSrv  *rel)
{
    Service *srv= rel->getFirst(true);

    if (rel->size()==1) {
	if (UDPService::isA(srv) || TCPService::isA(srv)) {
	    string str=_printDstPorts( srv );
	    if (! str.empty() ) output << "--destination-port " << str << " ";
	}
	if (ICMPService::isA(srv)) {
	    string str=_printICMP( ICMPService::cast(srv) );
	    if (! str.empty() ) output << "--icmp-type " << str << " ";
	}
	if (IPService::isA(srv)) {
	    string str=_printIP( IPService::cast(srv) );
	    if (! str.empty() ) output << str << " ";
	}

    } else {
/* use multiport */

	string str;
	bool   first=true;
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];

	    Service *s=Service::cast( o );
	    assert(s);
	    if (UDPService::isA(srv) || TCPService::isA(srv)) {
		if (!first) str+=",";
		str+= _printDstPorts( s );
		if (!str.empty()) first=false;
	    }
	}
	if ( !str.empty() ) {
	    output << "--destination-port ";
	    output << str << " ";
	}
    }
}

void NATCompiler_ipt::_printAddr(Address  *o,bool print_mask,bool print_range)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL) {
        if (Interface::cast(o)->isDyn()) return;
	mask=Netmask("255.255.255.255");
    }

    if (print_range && AddressRange::cast(o)!=NULL) {
	IPAddress a1=AddressRange::cast(o)->getRangeStart();
	IPAddress a2=AddressRange::cast(o)->getRangeEnd();
	output << a1.toString() << "-" << a2.toString();
    } else {
	if (addr.toString()=="0.0.0.0" && mask.toString()=="0.0.0.0") {
	    output << "0/0";
	} else {	
	    output << addr.toString();
	    if (print_mask && mask.toString()!="255.255.255.255") {
		output << "/" << mask.getLength();
	    }
	}
    }
}


void NATCompiler_ipt::PrintAtomicRule::operator()(NATRule *rule) const
{
    RuleElementOSrc *osrcrel=rule->getOSrc();
    Address  *osrc=osrcrel->getFirst(true);  assert(osrc);
    RuleElementODst *odstrel=rule->getODst();
    Address  *odst=odstrel->getFirst(true);  assert(odst);
    RuleElementOSrv *osrvrel=rule->getOSrv();
    Service  *osrv=osrvrel->getFirst(true);  assert(osrv);

    Address  *tsrc=rule->getTSrc()->getFirst(true);  assert(tsrc);
    Address  *tdst=rule->getTDst()->getFirst(true);  assert(tdst);
    Service  *tsrv=rule->getTSrv()->getFirst(true);  assert(tsrv);

    Interface *iface=
	Interface::cast( rule->getRoot()->getById(rule->getStr("interface_id") ,true) );


    getCompiler()->output  << "iptables -t nat ";
    getCompiler()->_printChainDirectionAndInterface(rule);

    if (! osrv->isAny() )
	getCompiler()->_printProtocol(osrv);

    getCompiler()->_printMultiport(rule);

    if (!osrc->isAny()) {
	getCompiler()->output << " -s ";
	getCompiler()->_printAddr(osrc);
    }

    if (!osrv->isAny()) {
	getCompiler()->output << " ";
	getCompiler()->_printSrcService(osrvrel);
    }

    if (!odst->isAny()) {
	getCompiler()->output << " -d ";
	getCompiler()->_printAddr(odst);
    }

    getCompiler()->output << " ";
    getCompiler()->_printDstService(osrvrel);

    getCompiler()->output << "-j " << rule->getStr("ipt_target") << " ";

    switch (rule->getRuleType()) {
    case NATRule::SNAT:  
	if (rule->getStr("ipt_target")=="SNAT") {
	    getCompiler()->output << "--to-source ";
	    getCompiler()->_printAddr(tsrc,false,true);
	    string ports=getCompiler()->_printSrcPorts(tsrv);
	    if (!ports.empty()) getCompiler()->output << ":" << ports;
	}
	break;

    case NATRule::DNAT:  
	if (rule->getStr("ipt_target")=="DNAT") {
	    getCompiler()->output << "--to-destination ";
	    getCompiler()->_printAddr(tdst,false,true);
	    string ports=getCompiler()->_printDstPorts(tsrv);
	    if (!ports.empty()) getCompiler()->output << ":" << ports;
	}
	break;

    case NATRule::SNetnat:
        getCompiler()->output << "--to ";
        getCompiler()->_printAddr(tsrc,true,false);
        break;

    case NATRule::DNetnat:
        getCompiler()->output << "--to ";
        getCompiler()->_printAddr(tdst,true,false);
        break;

    case NATRule::Redirect: 
	if (rule->getStr("ipt_target")=="REDIRECT") {
	    string ports=getCompiler()->_printDstPorts(tsrv);
	    if (!ports.empty()) getCompiler()->output << "--to-ports " << ports;
	}
	break;
    default:	break;
    }
    getCompiler()->output << " ";


    getCompiler()->output  << endl;

}

