/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_ipt_optimizer.cc,v 1.1 2002/05/08 01:26:10 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PolicyCompiler_ipt.hh"

#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Policy.hh"

#include <values.h>

#include <iostream.h>
#include <iomanip.h>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;


void PolicyCompiler_ipt::_optimizeForRuleElement(PolicyRule    *rule, 
                                                 vector<Rule*> *vr,
                                                 const std::string  &re_type)
{
    RuleElement    *re=RuleElement::cast(rule->getFirstByType(re_type));
    int nre=re->size();

    PolicyRule     *r;

    string new_chain=getNewTmpChainName(rule);

    r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
    temp_ruleset->add(r);
    r->duplicate(rule);

    for (FWObject::iterator i=r->begin(); i!=r->end(); ++i) {
        if (RuleElement::cast(*i)!=NULL && (*i)->getTypeName()!=re_type) {
            RuleElement *nre=RuleElement::cast(*i);
            nre->clearChildren();  
            nre->setAnyElement();
        }
    }
    r->setStr("ipt_target",new_chain);
    vr->push_back(r);

    rule->setStr("ipt_chain",new_chain);
    re->clearChildren();  
    re->setAnyElement();
    vr->push_back(rule);
}

void PolicyCompiler_ipt::optimize1::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementSrc *srcrel=rule->getSrc();
    RuleElementDst *dstrel=rule->getDst();
    RuleElementSrv *srvrel=rule->getSrv();

    int srcn=srcrel->size();
    int dstn=dstrel->size();
    int srvn=srvrel->size();

    if ( (srcn==1 && dstn==1) ||
         (dstn==1 && srvn==1) ||
         (srvn==1 && srcn==1) ) {
        vr->push_back(rule);
        return;
    }

    if (srcn==1) srcn=MAXINT;
    if (dstn==1) dstn=MAXINT;
    if (srvn==1) srvn=MAXINT;

    string re=RuleElementSrc::TYPENAME;

    if (srcn<=dstn && dstn<=srvn) {
        getCompiler()->_optimizeForRuleElement(rule,vr,RuleElementSrc::TYPENAME);
        return;
    }
    if (dstn<=srvn && srvn<=srcn) {
        getCompiler()->_optimizeForRuleElement(rule,vr,RuleElementDst::TYPENAME);
        return;
    }
    if (srvn<=srcn && srcn<=dstn) {
        getCompiler()->_optimizeForRuleElement(rule,vr,RuleElementSrv::TYPENAME);
        return;
    }

    vr->push_back(rule);
}
