/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_ipt_writers.cc,v 1.6 2002/05/04 01:32:15 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PolicyCompiler_ipt.hh"

#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/CustomService.hh"
#include "fwbuilder/Policy.hh"

#include "fwbuilder/Network.hh"

#include <iostream.h>
#include <streambuf.h>
#include <iomanip.h>
#include <fstream>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;



/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */
void PolicyCompiler_ipt::_checkAndCreateNewChain(PolicyRule *rule)
{
    string s;

    s=rule->getStr("ipt_chain");
    if ( ! chains[s] && ! standard_chains[s] ) {
	output << "iptables -N " << s << endl;
	chains[s]=true;
    }

    s=rule->getStr("ipt_target");
    if ( ! chains[s] && ! standard_chains[s] ) {
	output << "iptables -N " << s << endl;
	chains[s]=true;
    }
}

void PolicyCompiler_ipt::_printChain(PolicyRule *rule)
{
    string s=rule->getStr("ipt_chain");
    if (s.empty()) s="UNKNOWN";
    output << s << " ";
}

void PolicyCompiler_ipt::_printTarget(PolicyRule *rule)
{
    string s=rule->getStr("ipt_target");
    if (s.empty()) s="UNKNOWN";
    output << "-j " << s << " ";
    if (s=="LOG")    output << _printLogParameters(rule);
    if (s=="REJECT") output << _printActionOnReject(rule);
}

void PolicyCompiler_ipt::_printMultiport(PolicyRule *rule)
{
    if( rule->getBool("ipt_multiport"))
	output << "-m multiport ";
}

void PolicyCompiler_ipt::_printDirectionAndInterface(PolicyRule *rule)
{
    string       iface_id = rule->getStr("interface_id");
    if (rule->getDirection()==PolicyRule::Both) return;
    if (iface_id.empty() || iface_id=="nil" )   return;

    if (rule->getDirection()==PolicyRule::Inbound)   output << "-i "; 
    if (rule->getDirection()==PolicyRule::Outbound)  output << "-o  "; 
    Interface *rule_iface = fw_interfaces[iface_id];
    output << rule_iface->getName() << " ";
}

string PolicyCompiler_ipt::_printGlobalLogParameters()
{
    return _printLogParameters(NULL);
}

string PolicyCompiler_ipt::_printActionOnReject(libfwbuilder::PolicyRule *rule)
{
    std::ostringstream str;
    FWOptions *ruleopt =(rule!=NULL)?rule->getOptionsObject():fwopt;
    RuleElementSrv *srvrel=rule->getSrv();
    Service        *srv   =srvrel->getFirst(true);  assert(srv);

    string s;
    s=ruleopt->getStr("action_on_reject");
    if (s.empty()) s=fwopt->getStr("action_on_reject");
    if (!s.empty()) {
	if (s.find("TCP ")!=string::npos && srv->getProtocolName()=="tcp") 
	    str << "--reject-with tcp-reset";

	if (s.find("ICMP")!=string::npos) {
	    str << " --reject-with ";
	    if (s.find("unreachable")!=string::npos) {
		if (s.find("net")!=string::npos)   str << "icmp-net-unreachable";
		if (s.find("host")!=string::npos)  str << "icmp-host-unreachable";
		if (s.find("port")!=string::npos)  str << "icmp-port-unreachable";
		if (s.find("proto")!=string::npos) str << "icmp-proto-unreachable";
	    }
	    if (s.find("prohibited")!=string::npos) {
		if (s.find("net")!=string::npos)   str << "icmp-net-prohibited";
		if (s.find("host")!=string::npos)  str << "icmp-host-prohibited";
	    }
	}
    }
    str << " ";
    return str.str();
}

string PolicyCompiler_ipt::_printLogParameters(libfwbuilder::PolicyRule *rule)
{
    std::ostringstream str;
    string s;
    int    l;
    FWOptions *ruleopt =(rule!=NULL)?rule->getOptionsObject():fwopt;

    s=ruleopt->getStr("log_level");
    if (s.empty())  s=fwopt->getStr("log_level");
    if (!s.empty()) {
	if ( fwopt->getBool("use_numeric_log_levels") ) {
	    if (s=="alert")   s="1";
	    if (s=="crit")    s="2";
	    if (s=="error")   s="3";
	    if (s=="warning") s="4";
	    if (s=="notice")  s="5";
	    if (s=="info")    s="6";
	    if (s=="debug")   s="7";
	}
	str << " --log-level " << s;
    }

    s=ruleopt->getStr("log_prefix");
    if (s.empty())  s=fwopt->getStr("log_prefix");
    if (!s.empty()) {
/* deal with our logging macros:
 * %N - rule number
 * %A - action
 * %I - interface name
 * %C - chain name
 */
	string::size_type n;
	if (rule && (n=s.find("%N"))!=string::npos ) {
	    std::ostringstream s1;
	    s1 << rule->getPosition();
	    s.replace(n,2,s1.str());
	}
	if (rule && (n=s.find("%A"))!=string::npos ) {
	    std::ostringstream s1;
	    switch (rule->getAction()) {
	    case PolicyRule::Accept:  s1 << "ACCEPT"; break;
	    case PolicyRule::Deny:    s1 << "DROP";   break;
	    case PolicyRule::Reject:  s1 << "REJECT"; break;
	    case PolicyRule::Return:  s1 << "RETURN"; break;
	    }
	    s.replace(n,2,s1.str());
	}
	if (rule && (n=s.find("%I"))!=string::npos ) {
	    std::ostringstream s1;
	    Interface *rule_iface = fw_interfaces[rule->getStr("interface_id")];
	    if (rule_iface!=NULL) {
		s1 << rule_iface->getName();
		s.replace(n,2,s1.str());
	    }
	}
	if (rule && (n=s.find("%C"))!=string::npos ) {
	    s.replace(n,2,rule->getStr("ipt_chain"));
	}
	str << " --log-prefix \"" << s << "\"";
    }

    if (ruleopt->getBool("log_tcp_seq") || fwopt->getBool("log_tcp_seq"))  
	str << " --log-tcp-sequence ";
    if (ruleopt->getBool("log_tcp_opt") || fwopt->getBool("log_tcp_opt"))  
	str << " --log-tcp-options ";
    if (ruleopt->getBool("log_ip_opt") || fwopt->getBool("log_ip_opt"))   
	str << " --log-ip-options ";

    if ( (l=ruleopt->getInt("limit_value"))>0 || (l=fwopt->getInt("limit_value"))>0 ) {
	str << "  -m limit --limit " << l;

	s=ruleopt->getStr("limit_suffix");
	if (s.empty()) 	s=fwopt->getStr("limit_suffix");
	if (!s.empty()) str << s;

	s=ruleopt->getStr("limit_burst");
	if (s.empty()) fwopt->getStr("limit_burst");
	if (!s.empty()) str << " --limit-burst" << s;
    }
    return str.str();
}

void PolicyCompiler_ipt::_printProtocol(libfwbuilder::Service *srv)
{
    if ( ! srv->isAny() && ! CustomService::isA(srv) ) {
        output << "-p " <<  srv->getProtocolName() << " ";
    }
}

string PolicyCompiler_ipt::_printPorts(int rs,int re)
{
    std::ostringstream  str;

    normalizePortRange(rs,re);

    if (rs>0 || re>0) {
	if (rs==re)  str << rs;
	else
	    if (rs==0 && re!=0)      str << ":" << re;
	    else
		if (rs!=0 && re==65535)  str << rs << ":";
		else {
		    str << rs << ":" << re;
		}
    }
    return str.str();
}

string PolicyCompiler_ipt::_printSrcPorts(Service *srv)
{
    std::ostringstream  str;
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int rs=srv->getInt("src_range_start");
	int re=srv->getInt("src_range_end");
	str << _printPorts(rs,re);
    }
    return str.str();
}

string PolicyCompiler_ipt::_printDstPorts(Service *srv)
{
    std::ostringstream  str;
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int rs=srv->getInt("dst_range_start");
	int re=srv->getInt("dst_range_end");
	str << _printPorts(rs,re);
    }
    return str.str();
}

string PolicyCompiler_ipt::_printICMP(ICMPService *srv)
{
    std::ostringstream  str;
    if (ICMPService::isA(srv) && srv->getInt("type")!=-1) {
	str << srv->getStr("type");
	if (srv->getInt("code")!=-1) 
	    str << "/" << srv->getStr("code") << " ";
    }
    return str.str();
}

string PolicyCompiler_ipt::_printIP(IPService *srv)
{
    std::ostringstream  str;
    if (IPService::isA(srv) ) {
	if (srv->getBool("fragm") || srv->getBool("short_fragm"))
	    str << "-f ";

        if  (srv->getBool("lsrr") ||
             srv->getBool("ssrr") ||
             srv->getBool("rr") ||
             srv->getBool("ts") ) str << " -m ipv4options ";

        if  (srv->getBool("lsrr")) str << " --lsrr";
        if  (srv->getBool("ssrr")) str << " --ssrr";
        if  (srv->getBool("rr"))   str << " --rr";
        if  (srv->getBool("ts"))   str << " --ts";
    }
    return str.str();
}

string PolicyCompiler_ipt::_printTCPFlags(libfwbuilder::TCPService *srv)
{
    string str;
    bool   first=true;

    if (srv->getSYNFlag() && 
	! srv->getURGFlag() && 
	! srv->getACKFlag() && 
	! srv->getPSHFlag() && 
	! srv->getRSTFlag() && 
	! srv->getFINFlag()) {

	str="--syn ";

    } else {

	if (srv->getSYNFlag() ||
	    srv->getURGFlag() || 
	    srv->getACKFlag() || 
	    srv->getPSHFlag() || 
	    srv->getRSTFlag() || 
	    srv->getFINFlag()) str="--tcp-flags ALL ";

	if (srv->getURGFlag()) { if (!first) str+=","; str+="URG"; first=false; }
	if (srv->getACKFlag()) { if (!first) str+=","; str+="ACK"; first=false; }
	if (srv->getPSHFlag()) { if (!first) str+=","; str+="PSH"; first=false; }
	if (srv->getRSTFlag()) { if (!first) str+=","; str+="RST"; first=false; }
	if (srv->getSYNFlag()) { if (!first) str+=","; str+="SYN"; first=false; }
	if (srv->getFINFlag()) { if (!first) str+=","; str+="FIN"; first=false; }
    }

    return str;
}

/*
 * we made sure that all services in rel  represent the same protocol
 */
void PolicyCompiler_ipt::_printSrcService(RuleElementSrv  *rel)
{
    Service *srv= rel->getFirst(true);

    if (rel->size()==1) {
	if (UDPService::isA(srv) || TCPService::isA(srv)) {
	    string str=_printSrcPorts( srv );
	    if (! str.empty() ) output << "--source-port " << str << " ";
	}
    } else {
/* use multiport */

	string str;
	bool  first=true;
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];

	    Service *s=Service::cast( o );
	    assert(s);
	    if (UDPService::isA(srv) || TCPService::isA(srv)) {
		if (!first) str+=",";
		str+= _printSrcPorts( s );
		if (!str.empty()) first=false;
	    }
	}
	if ( !str.empty() ) {
	    output << "--source-port ";
	    output << str << " ";
	}
    }
}

void PolicyCompiler_ipt::_printDstService(RuleElementSrv  *rel)
{
    Service *srv= rel->getFirst(true);

    if (rel->size()==1) {
	if (UDPService::isA(srv) || TCPService::isA(srv)) {
	    string str=_printDstPorts( srv );
	    if (! str.empty() ) output << "--destination-port " << str << " ";
	}
	if (TCPService::isA(srv)) {
	    string str=_printTCPFlags(TCPService::cast(srv));
	    if (!str.empty()) output << str << " ";
	}
	if (ICMPService::isA(srv)) {
	    string str=_printICMP( ICMPService::cast(srv) );
	    if (! str.empty() ) output << "--icmp-type " << str << " ";
	}
	if (IPService::isA(srv)) {
	    string str=_printIP( IPService::cast(srv) );
	    if (! str.empty() ) output << str << " ";
	}
	if (CustomService::isA(srv)) {
	    output << CustomService::cast(srv)->getCodeForPlatform( myPlatformName() ) << " ";
	}
    } else {
/* use multiport */

	string str;
	bool   first=true;
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    FWObject *o= *i;
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];

	    Service *s=Service::cast( o );
	    assert(s);
	    if (UDPService::isA(srv) || TCPService::isA(srv)) {
		if (!first) str+=",";
		str+= _printDstPorts( s );
		if (!str.empty()) first=false;
	    }
	}
	if ( !str.empty() ) {
	    output << "--destination-port ";
	    output << str << " ";
	}
    }
}

void PolicyCompiler_ipt::_printAddr(Address  *o)
{
    IPAddress addr;
    Netmask   mask;

    try {
        addr=o->getAddress();
        mask=o->getNetmask();
    }
    catch (FWException ex)  {
        FWObject *obj=o;
/*
 * check if this is object of class Address. since we want to
 * distinguish between Host, Interface and Address, and both Host and
 * Interface are inherited from Address, we can't use cast. Use isA
 * instead
 */
        while (obj!=NULL && 
               !Host::isA(obj) && 
               !Firewall::isA(obj)  && 
               !Network::isA(obj))  obj=o->getParent();

        error("Problem with address or netmask in the object or one of its interfaces: '"+obj->getName()+"'");
        throw;
    }

    if (Interface::cast(o)!=NULL) 
	mask=Netmask("255.255.255.255");


    if (addr.toString()=="0.0.0.0" && mask.toString()=="0.0.0.0") {
	output << "0/0 ";
    } else {	
	output << addr.toString();
	if (mask.toString()!="255.255.255.255") {
	    output << "/" << mask.getLength();
	}
	output << " ";
    }
}

void PolicyCompiler_ipt::_printSrcAddr(RuleElementSrc  *rel)
{
    if (rel->size()==1) {
	_printAddr( rel->getFirst(true) );
    } else {

	output << "{ ";
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    if (i!=rel->begin())  output << ", ";
	    FWObject *o= *i;
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];

	    Address *s=Address::cast( o );
	    assert(s);
	    _printAddr( s );
	}
	output << "} ";
    }

}

void PolicyCompiler_ipt::_printDstAddr(RuleElementDst  *rel)
{
    if (rel->size()==1) {
	_printAddr( rel->getFirst(true) );
    } else {

	output << "{ ";
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    if (i!=rel->begin())  output << ", ";
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];

	    Address *s=Address::cast( o );
	    assert(s);
	    _printAddr( s );
	}
	output << "} ";
    }

}

void PolicyCompiler_ipt::PrintRule::operator()(PolicyRule *rule) const
{
    FWOptions *ruleopt =rule->getOptionsObject();

    RuleElementSrc *srcrel=rule->getSrc();
    Address        *src   =srcrel->getFirst(true);  assert(src);
    RuleElementDst *dstrel=rule->getDst();
    Address        *dst   =dstrel->getFirst(true);  assert(dst);
    RuleElementSrv *srvrel=rule->getSrv();
    Service        *srv   =srvrel->getFirst(true);  assert(srv);

    getCompiler()->output << "iptables -A ";

    getCompiler()->_printChain(rule);
    getCompiler()->_printDirectionAndInterface(rule);

    if (! srv->isAny() )
	getCompiler()->_printProtocol(srv);

    getCompiler()->_printMultiport(rule);

    if (!src->isAny()) {
	Interface *iface=Interface::cast(src);
	if (iface!=NULL && iface->getBool("use_mac_addr")) {
/* processor prepareForMACMatching should have replaced src with references
 * to host's interfaces, so iface!=NULL 
 */
	    getCompiler()->output << " -m mac --mac-source " << iface->getPhysicalAddress();

	    if ( iface->getAddress().to32BitInt()!=0 ) {
		getCompiler()->output << " -s ";
		getCompiler()->_printAddr(src);
	    }
	} else {
	    getCompiler()->output << " -s ";
	    getCompiler()->_printAddr(src);
	}
    }
    getCompiler()->_printSrcService(srvrel);

    if (!dst->isAny()) {
	getCompiler()->output << " -d ";
	getCompiler()->_printAddr(dst);
    }
    getCompiler()->_printDstService(srvrel);

/* keeping state does not apply to deny/reject */
    if ( ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept) {  
	getCompiler()->output << "-m state --state NEW ";
    }

    getCompiler()->_printTarget(rule);

    getCompiler()->output << endl;
}

