/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_ipf.cc,v 1.17 2002/05/13 04:13:51 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "NATCompiler_ipf.hh"

#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/NAT.hh"
#include "fwbuilder/Network.hh"

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;

string NATCompiler_ipf::myPlatformName() { return "ipf"; }


int NATCompiler_ipf::prolog()
{

    int n=NATCompiler_pf::prolog();

    return n;
}

void NATCompiler_ipf::VerifyRules::operator()(NATRule *rule) const
{
    RuleElementOSrc  *osrc=rule->getOSrc();  assert(osrc);
    RuleElementODst  *odst=rule->getODst();  assert(odst);
    RuleElementOSrv  *osrv=rule->getOSrv();  assert(osrv);

    RuleElementTSrc  *tsrc=rule->getTSrc();  assert(tsrc);
    RuleElementTDst  *tdst=rule->getTDst();  assert(tdst);
    RuleElementTSrv  *tsrv=rule->getTSrv();  assert(tsrv);

    if (rule->getRuleType()==NATRule::DNAT && odst->size()!=1)
	throw FWException("There should be no more than one object in original destination in the rule "+rule->getStr("rule_label"));

    if (rule->getRuleType()==NATRule::SNAT && tsrc->size()!=1)
	throw FWException("There should be no more than one object in translated source in the rule "+rule->getStr("rule_label"));

    if (rule->getRuleType()==NATRule::DNAT && osrv->isAny()) 
	throw FWException("Service must be specified for destination translation rule. Rule "+rule->getStr("rule_label"));

    if (tsrv->size()!=1) 
	throw FWException("Translated service should be 'Original' or should contain single object. Rule: "+rule->getStr("rule_label"));

    FWObject *o=tsrv->front();
    if (FWReference::cast(o)!=NULL) o=getCompiler()->objcache[o->getStr("ref")];

    if ( Group::cast(o)!=NULL)
	throw FWException("Can not use group in translated service. Rule "+rule->getStr("rule_label"));


    if (rule->getRuleType()==NATRule::SNAT ) {
        if ( tsrc->size()!=1)
            throw FWException("There should be no more than one object in translated source in the rule "+rule->getStr("rule_label"));

//        Address* o1=tsrc->getFirst(true);
//        if ( ! tsrc->isAny() && Network::cast(o1)!=NULL)
//            throw FWException("Can not use network object in translated source. Rule "+rule->getStr("rule_label"));
    }


    if (rule->getRuleType()==NATRule::SNetnat && !tsrc->isAny() ) {
        Network *a1=Network::cast(osrc->getFirst(true));
        Network *a2=Network::cast(tsrc->getFirst(true));
        if ( a1==NULL || a2==NULL ||
             a1->getNetmask().getLength()!=a2->getNetmask().getLength() )
            throw FWException("Original and translated source should both be networks of the same size . Rule "+rule->getStr("rule_label"));
    }

    if (rule->getRuleType()==NATRule::DNetnat && !tsrc->isAny() ) {
        Network *a1=Network::cast(odst->getFirst(true));
        Network *a2=Network::cast(tdst->getFirst(true));
        if ( a1==NULL || a2==NULL ||
             a1->getNetmask().getLength()!=a2->getNetmask().getLength() )
            throw FWException("Original and translated destination should both be networks of the same size . Rule "+rule->getStr("rule_label"));
    }




    if (osrc->getNeg() || odst->getNeg() || osrv->getNeg())
        throw FWException("Negation in NAT rules is not supported. Rule "+rule->getStr("rule_label"));

//    if (rule->getRuleType()==NATRule::SNAT && ! odst->isAny() )
//	getCompiler()->warning("Original destination is ignored in NAT rule "+rule->getStr("rule_label"));
	
    if (rule->getRuleType()==NATRule::DNAT && ! osrc->isAny() )
	getCompiler()->warning("Original source is ignored in NAT rule "+rule->getStr("rule_label"));

    if (rule->getRuleType()==NATRule::NONAT)
        throw FWException("Unsupported translation. Rule "+rule->getStr("rule_label"));
    
}

void NATCompiler_ipf::appProxy::operator()(NATRule *rule, vector<Rule*> *vr) const
{
    bool ftp_proxy    = getCompiler()->fwopt->getBool("ipf_nat_ftp_proxy");
    bool rcmd_proxy   = getCompiler()->fwopt->getBool("ipf_nat_rcmd_proxy");
    bool raudio_proxy = getCompiler()->fwopt->getBool("ipf_nat_raudio_proxy");

    if (rule->getRuleType()==NATRule::SNAT) {

	if (ftp_proxy) {
	    NATRule *r= NATRule::cast(getCompiler()->
				      dbcopy->create(NATRule::TYPENAME,true) );
	    getCompiler()->temp_ruleset->add(r);
	    r->duplicate(rule);
	    r->setBool("ftp_proxy",true);
	    vr->push_back( r );
	}

	if (rcmd_proxy) {
	    NATRule *r= NATRule::cast(getCompiler()->
				      dbcopy->create(NATRule::TYPENAME,true) );
	    getCompiler()->temp_ruleset->add(r);
	    r->duplicate(rule);
	    r->setBool("rcmd_proxy",true);
	    vr->push_back( r );
	}

	if (raudio_proxy) {
	    NATRule *r= NATRule::cast(getCompiler()->
				      dbcopy->create(NATRule::TYPENAME,true) );
	    getCompiler()->temp_ruleset->add(r);
	    r->duplicate(rule);
	    r->setBool("raudio_proxy",true);
	    vr->push_back( r );
	}
    }
    vr->push_back(rule);
}


void NATCompiler_ipf::compile()
{
    bool manage_virtual_addr=fwopt->getBool("manage_virtual_addr");

    cout << " Compiling NAT rules for " << fw->getName() << " ..." << endl << flush;

    try {

	Compiler::compile();

	if (verbose) cout << "expand groups ...\n" << flush;
	for_each_rule( ExpandGroups(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rule on original service ..\n" << flush;
	for_each_rule( splitOnOSrv(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "fill translated service ..\n" << flush;
	for_each_rule( fillTranslatedSrv(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "determine NAT rule types ..\n" << flush;
	for_each_rule( NATRuleType(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "verify NAT rules ..\n" << flush;
	for_each_rule( VerifyRules(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "replace references to the firewall ...\n" << flush;
	for_each_rule( ReplaceFirewallObjectsODst(this) );
	for_each_rule( ReplaceFirewallObjectsTSrc(this) );
	for_each_rule( ReplaceObjectsTDst(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if ( manage_virtual_addr ) {
	    if (verbose) cout << "add virtual addresses for NAT rules ...\n" << flush;
	    for_each_rule( addVirtualAddress(this) );
            if (debug_rule>=0) for_each_rule( debugRule(this) );
	}

	if (verbose) cout << "expand multiple addresses ...\n" << flush;
	for_each_rule( ExpandMultipleAddresses(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "expand address range objects  ...\n" << flush;
	for_each_rule( addressRanges(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "convert to atomic rules ...\n" << flush;
	for_each_rule( ConvertToAtomic(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "add application proxy code to map rules ...\n" << flush;
	for_each_rule( appProxy(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "assign rules to interfaces ...\n" << flush;
	for_each_rule( AssignInterface(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

    } catch (FWException &ex) {
	abort(ex.toString());
    }
}


void NATCompiler_ipf::epilog()
{
    try {
	if (verbose) cout << "printing ... \n" << flush;

/* print access lists here */
	PrintAtomicRule par(this);

	string current_rule_label="";

	for (list<FWObject*>::iterator k=combined_ruleset->begin(); k!=combined_ruleset->end(); ++k) {
	    NATRule *r = NATRule::cast( *k );
	    string rl=r->getStr("rule_label");
	    if (rl!=current_rule_label) {

                if (verbose) {
                    cout << "Rule " << rl << endl << flush;
                }

		output << "# " << endl;
		output << "# Rule  " << rl << endl;

                string    comm=r->getComment();
                string::size_type c1,c2;
                c1=0;
                while ( (c2=comm.find('\n',c1))!=string::npos ) {
                    output << "# " << comm.substr(c1,c2-c1) << endl;
                    c1=c2+1;
                }
                output << "# " << comm.substr(c1) << endl;
                output << "# " << endl;

		current_rule_label=rl;
	    }
	    par(r);
	}

	cout << " NAT rules compiled successfully \n" << flush;

    } catch (FWException ex) {
	abort(ex.toString());
    }
}
