/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_ipf.hh,v 1.8 2002/05/12 07:47:42 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef _NATCOMPILER_IPF_HH
#define _NATCOMPILER_IPF_HH

#include "NATCompiler_pf.hh"

#include <map>

namespace fwcompiler {


    class NATCompiler_ipf : public NATCompiler_pf {



	protected:

	virtual void _printProtocol(libfwbuilder::Service *srv);
	virtual void _printAddr(libfwbuilder::Address  *o, bool print_netmask=true);

	virtual std::string myPlatformName();


	/**
	 * this inspector verifies correctness of the NAT rules
	 * (some checks are the same as in pf, some are specific for ipf)
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_ipf,  VerifyRules );

	/**
	 * this processor splits NAT rules if user ordered using
	 * application proxy code for "map" rules
	 */
	DECLARE_NAT_RULE_PROCESSOR( NATCompiler_ipf,  appProxy );

	/**
	 *  this inspector prints single policy rule, assuming all
	 *  groups have been expanded, so source, destination and
	 *  service hold exactly one object each, and this object is
	 *  not a group.  Negation should also have been taken care of
	 *  before this method is called.
	 */
 	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_ipf,  PrintAtomicRule );


	public:

	NATCompiler_ipf(libfwbuilder::FWObjectDatabase *_db,
			const std::string &fwname,
			fwcompiler::OSConfigurator *_oscnf) : NATCompiler_pf(_db,fwname,_oscnf) {}


	virtual int  prolog();
	virtual void compile();
	virtual void epilog();
	
    };


}

#endif
