/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_ipf_writers.cc,v 1.2 2002/05/12 07:47:42 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "NATCompiler_ipf.hh"

#include "fwbuilder/AddressRange.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Policy.hh"

#include <iostream.h>
#include <streambuf.h>
#include <iomanip.h>
#include <fstream>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;



/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */





void NATCompiler_ipf::_printAddr(Address  *o, bool print_netmask)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL && ! Interface::cast(o)->isDyn()) 
	mask=Netmask("255.255.255.255");

    output << addr.toString();
    if (print_netmask)
	output << "/" << mask.getLength();
    output  << " ";
}

void NATCompiler_ipf::_printProtocol(Service *srv)
{
    output << srv->getProtocolName() << " ";
}


void NATCompiler_ipf::PrintAtomicRule::operator()(NATRule *rule) const
{
    Address  *osrc=rule->getOSrc()->getFirst(true);  assert(osrc);
    Address  *odst=rule->getODst()->getFirst(true);  assert(odst);
    Service  *osrv=rule->getOSrv()->getFirst(true);  assert(osrv);

    Address  *tsrc=rule->getTSrc()->getFirst(true);  assert(tsrc);
    Address  *tdst=rule->getTDst()->getFirst(true);  assert(tdst);
    Service  *tsrv=rule->getTSrv()->getFirst(true);  assert(tsrv);

    Interface *iface=
	Interface::cast( rule->getRoot()->getById(rule->getStr("interface_id") ,true) );

    if (rule->getRuleType()==NATRule::SNAT) {
	getCompiler()->output  << "map "
			       << iface->getName() 
			       << " ";
	getCompiler()->output  << "from ";
	getCompiler()->_printAddr( osrc );
	getCompiler()->output  << "to ";
	getCompiler()->_printAddr( odst );

	getCompiler()->output  << "-> ";
	getCompiler()->_printAddr( tsrc );
	getCompiler()->output  << " ";

	if (rule->getBool("ftp_proxy"))    getCompiler()->output << "proxy port ftp ftp/tcp ";
	if (rule->getBool("rcmd_proxy"))   getCompiler()->output << "proxy port rcmd rcmd/tcp ";
	if (rule->getBool("raudio_proxy")) getCompiler()->output << "proxy port raudio raudio/tcp ";

	getCompiler()->output  << endl;
    }

    if (rule->getRuleType()==NATRule::DNAT) {
	getCompiler()->output  << "rdr " 
			       << iface->getName() 
			       << " ";
	getCompiler()->_printAddr( odst , true );
	getCompiler()->_printPort(osrv);
	getCompiler()->output  << "-> ";
	getCompiler()->_printAddr( tdst , false );
	getCompiler()->_printPort(tsrv);
        if ( ! osrv->isAny()) getCompiler()->_printProtocol(osrv);
	getCompiler()->output  << endl;
    }

    if (rule->getRuleType()==NATRule::Redirect) {
	getCompiler()->output  << "rdr " 
			       << iface->getName() 
			       << " ";
	getCompiler()->_printAddr( odst , true );
	getCompiler()->_printPort(osrv);
	getCompiler()->output  << "-> ";
	getCompiler()->_printAddr( tdst , false );
	getCompiler()->_printPort(tsrv);
	getCompiler()->_printProtocol(osrv);
	getCompiler()->output  << endl;
    }
}

