/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_pf.hh,v 1.16 2002/05/10 03:09:34 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef _NATCOMPILER_PF_HH
#define _NATCOMPILER_PF_HH

#include "fwcompiler/NATCompiler.hh"

#include <map>

namespace libfwbuilder {
    class Host;
};

namespace fwcompiler {


    class NATCompiler_pf : public NATCompiler {

	protected:

	libfwbuilder::Host  *loopback_address;

	virtual void _printProtocol(libfwbuilder::Service *srv);
	virtual void _printPort(libfwbuilder::Service *srv);
	virtual void _printAddr(libfwbuilder::Address  *o, bool print_netmask=true);

        virtual string _internalPrintNATRule(libfwbuilder::NATRule *rule);


	/**
	 * this inspector determines type of the NAT rule
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf,  NATRuleType );

	/**
	 * this inspector verifies correctness of the NAT rules
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf,  VerifyRules );

	/**
	 * this processor splits rule with multiple service objects in OSrv
	 * onto several rules
	 */
	DECLARE_NAT_RULE_PROCESSOR( NATCompiler_pf,  splitOnOSrv );

	/**
	 * this inspector fills translated service with the copy of original srv
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf,  fillTranslatedSrv );

	/**
	 * this processor assigns NAT rules to interfaces 
	 */
	DECLARE_NAT_RULE_PROCESSOR( NATCompiler_pf,  AssignInterface );

	/**
	 * this inspector calls OSConfigurator to add virtual
	 * address to the firewall if it is needed for NAT rule
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf,  addVirtualAddress );

	/**
	 *  this inspector replaces references to the firewall in odst
	 *  with references to its external interfaces
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf, ReplaceFirewallObjectsODst );

	/**
	 *  this inspector replaces references to the firewall in tsrc
	 *  with references to its external interfaces
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf, ReplaceFirewallObjectsTSrc );

	/**
	 *  this inspector replaces object in tdst with reference to
	 *  pseudo-host with address 127.0.0.1
	 */
	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf, ReplaceObjectsTDst );

        /**
         * this processor deals with negation in OSrc 
         */
        DECLARE_NAT_RULE_PROCESSOR( NATCompiler_pf, doOSrcNegation );

        /**
         * this processor deals with negation in ODst
         */
        DECLARE_NAT_RULE_PROCESSOR( NATCompiler_pf, doODstNegation );

        /**
         * this processor deals with negation in OSrv 
         */
        DECLARE_NAT_RULE_PROCESSOR( NATCompiler_pf, doOSrvNegation );


	/**
	 *  this inspector prints single policy rule, assuming all
	 *  groups have been expanded, so source, destination and
	 *  service hold exactly one object each, and this object is
	 *  not a group.  Negation should also have been taken care of
	 *  before this method is called.
	 */
 	DECLARE_NAT_RULE_INSPECTOR( NATCompiler_pf,  PrintAtomicRule );


	virtual std::string myPlatformName();



	public:

	NATCompiler_pf(libfwbuilder::FWObjectDatabase *_db,
		       const std::string &fwname,
		       fwcompiler::OSConfigurator *_oscnf) : NATCompiler(_db,fwname,_oscnf) {}


	virtual int  prolog();
	virtual void compile();
	virtual void epilog();
	
//	virtual string atomicRuleToString(libfwbuilder::Rule *r);
    };


}

#endif
