/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_pf_negation.cc,v 1.1 2002/04/18 06:39:48 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "NATCompiler_pf.hh"

#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/Policy.hh"

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;




/*
 *   negation in OSrc :
 *
 *   !A    B    C    RULE_TYPE  
 *-----------------------------------------------
 *
 *    A    B    C    Continue ("no nat")
 *   any   B    C    SNAT/DNAT 
 */
void NATCompiler_pf::doOSrcNegation::operator()(NATRule *rule, vector<Rule*> *vr) const
{
    RuleElementOSrc *osrcrel=rule->getOSrc();

    if (osrcrel->getNeg()) {
	NATRule     *r;

	osrcrel->setNeg(false);

	r= NATRule::cast(getCompiler()->dbcopy->create(NATRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setRuleType(NATRule::Continue);
	vr->push_back(r);

	r= NATRule::cast(getCompiler()->dbcopy->create(NATRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	RuleElementOSrc *nsrc=r->getOSrc();  nsrc->clearChildren();  nsrc->setAnyElement();
	vr->push_back(r);
    } else
	vr->push_back(rule);
}

/*
 *   negation in Odst :
 *
 *   A    !B   C    RULE_TYPE  TARGET
 *-------------------------------------
 *
 *   A    B    C    Continue ("no nat")
 *   A   any   C    SNAT/DNAT 
 */
void NATCompiler_pf::doODstNegation::operator()(NATRule *rule, vector<Rule*> *vr) const
{
    RuleElementODst *odstrel=rule->getODst();

    if (odstrel->getNeg()) {
	NATRule     *r;

	odstrel->setNeg(false);

	r= NATRule::cast(getCompiler()->dbcopy->create(NATRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setRuleType(NATRule::Continue);
	vr->push_back(r);

	r= NATRule::cast(getCompiler()->dbcopy->create(NATRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	RuleElementODst *ndst=r->getODst();  ndst->clearChildren();  ndst->setAnyElement();
	vr->push_back(r);
    } else
	vr->push_back(rule);
}
/*
 *   negation in OSrv :
 *
 *   A    B   !C    RULE_TYPE  TARGET
 *-------------------------------------
 * 
 *   A    B    C    Continue ("no nat")
 *   A    B   any   SNAT/DNAT 
 */
void NATCompiler_pf::doOSrvNegation::operator()(NATRule *rule, vector<Rule*> *vr) const
{
    RuleElementOSrv *osrvrel=rule->getOSrv();

/*   A  B ! C    */

    if (osrvrel->getNeg()) {
	NATRule     *r;

	osrvrel->setNeg(false);

	r= NATRule::cast(getCompiler()->dbcopy->create(NATRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setRuleType(NATRule::Continue);
	vr->push_back(r);

	r= NATRule::cast(getCompiler()->dbcopy->create(NATRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	RuleElementOSrv *nsrv=r->getOSrv();  nsrv->clearChildren();  nsrv->setAnyElement();
	vr->push_back(r);
    } else
	vr->push_back(rule);
}

