/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: NATCompiler_pf_writers.cc,v 1.2 2002/05/10 03:09:34 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "NATCompiler_pf.hh"

#include "fwbuilder/AddressRange.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Policy.hh"

#include <iostream.h>
#include <streambuf.h>
#include <iomanip.h>
#include <fstream>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;



/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */




void NATCompiler_pf::PrintAtomicRule::operator()(NATRule *rule) const
{
    Address  *osrc=rule->getOSrc()->getFirst(true);  assert(osrc);
    Address  *odst=rule->getODst()->getFirst(true);  assert(odst);
    Service  *osrv=rule->getOSrv()->getFirst(true);  assert(osrv);

    Address  *tsrc=rule->getTSrc()->getFirst(true);  assert(tsrc);
    Address  *tdst=rule->getTDst()->getFirst(true);  assert(tdst);
    Service  *tsrv=rule->getTSrv()->getFirst(true);  assert(tsrv);

    Interface *iface=
	Interface::cast( rule->getRoot()->getById(rule->getStr("interface_id") ,true) );

    switch ( rule->getRuleType() ) {
    case NATRule::Continue:
    case NATRule::NONAT:
	getCompiler()->output  << "no nat on " 
			       << iface->getName() 
			       << " ";
	getCompiler()->_printProtocol(osrv);
	getCompiler()->output  << "from ";
	getCompiler()->_printAddr( osrc );
	getCompiler()->output  << "to ";
	getCompiler()->_printAddr( odst );

	getCompiler()->output  << endl;
        break;

    case NATRule::SNAT:
	getCompiler()->output  << "nat on " 
			       << iface->getName() 
			       << " ";
	getCompiler()->_printProtocol(osrv);
	getCompiler()->output  << "from ";
	getCompiler()->_printAddr( osrc );
	getCompiler()->output  << "to ";
	getCompiler()->_printAddr( odst );

	getCompiler()->output  << "-> ";
	getCompiler()->_printAddr( tsrc , false );

	getCompiler()->output  << endl;
        break;

    case NATRule::DNAT:
	getCompiler()->output  << "rdr on " 
			       << iface->getName() 
			       << " ";
	getCompiler()->_printProtocol(osrv);
	getCompiler()->output  << "from ";
	getCompiler()->_printAddr( osrc );
	getCompiler()->output  << "to ";
	getCompiler()->_printAddr( odst );
	getCompiler()->_printPort(osrv);
	getCompiler()->output  << "-> ";
	getCompiler()->_printAddr( tdst , false );
	getCompiler()->_printPort(tsrv);
	getCompiler()->output  << endl;
        break;

    case NATRule::Redirect:
	getCompiler()->output  << "rdr on " 
			       << iface->getName() 
			       << " ";
	getCompiler()->_printProtocol(osrv);
	getCompiler()->output  << "from ";
	getCompiler()->_printAddr( osrc );
	getCompiler()->output  << "to ";
	getCompiler()->_printAddr( odst );
	getCompiler()->_printPort(osrv);
	getCompiler()->output  << "-> ";
	getCompiler()->_printAddr( tdst , false );
	getCompiler()->_printPort(tsrv);
	getCompiler()->output  << endl;
        break;
    }
}

void NATCompiler_pf::_printProtocol(Service *srv)
{
    output << "proto " <<  srv->getProtocolName() << " ";
}

void NATCompiler_pf::_printPort(Service *srv)
{
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int drs=srv->getInt("dst_range_start");
	if (drs!=0)   output << "port " << drs << " ";
    }
}


void NATCompiler_pf::_printAddr(Address  *o, bool print_netmask)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL) {
	Interface *interface_=Interface::cast(o);
	if (interface_->isDyn()) {
	    output << interface_->getName() << " ";
	    return;
	}

	mask=Netmask("255.255.255.255");
    }

    if (addr.toString()=="0.0.0.0" && mask.toString()=="0.0.0.0") {
	output << "any ";
    } else {
	output << addr.toString();
	if (print_netmask && mask.toString()!="255.255.255.255") {
	    output << "/" << mask.getLength();
	}
	output  << " ";
    }
}

