/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: OSConfigurator_openbsd.cc,v 1.4 2002/04/24 05:08:41 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifdef __MINGW32__
#include <assert.h>
#endif

#include "OSConfigurator_openbsd.hh"

#include <algorithm>
#include <functional>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;

string OSConfigurator_openbsd::myPlatformName() { return "OpenBSD"; }

void OSConfigurator_openbsd::processFirewallOptions() 
{
    FWOptions* options=fw->getOptionsObject();
    string s;

    s=options->getStr("openbsd_ip_directed_broadcast");
    if (!s.empty()) {
	if (s!="0" && s!="1")
	    throw FWException("Illegal value for OS parameter openbsd_ip_directed_broadcast: '"+s+"'");

	output << "sysctl -w net.inet.ip.directed-broadcast=" << s << endl;
    }

    s=options->getStr("openbsd_ip_forward");
    if (!s.empty()) {
	if (s!="0" && s!="1")
	    throw FWException("Illegal value for OS parameter openbsd_ip_forward: '"+s+"'");

	output << "sysctl -w net.inet.ip.forwarding=" << s << endl;
    }

    s=options->getStr("openbsd_ip_sourceroute");
    if (!s.empty()) {
	if (s!="0" && s!="1")
	    throw FWException("Illegal value for OS parameter openbsd_ip_sourceroute: '"+s+"'");

	output << "sysctl -w net.inet.ip.sourceroute=" << s << endl;
    }

    s=options->getStr("openbsd_ip_redirect");
    if (!s.empty()) {
	if (s!="0" && s!="1")
	    throw FWException("Illegal value for OS parameter openbsd_ip_redirect: '"+s+"'");

	output << "sysctl -w net.inet.ip.redirect=" << s << endl;
    }
}

void OSConfigurator_openbsd::addVirtualAddressForNAT(const IPAddress &addr)
{
    if (virtual_addresses.empty() || 
	find(virtual_addresses.begin(),virtual_addresses.end(),addr)==virtual_addresses.end()) {

	list<FWObject*> l2=fw->getByType(Interface::TYPENAME);
	for (list<FWObject*>::iterator i=l2.begin(); i!=l2.end(); ++i) {
	    Interface *iface=dynamic_cast<Interface*>(*i);
	    assert(iface);

/* this is the network interface is sitting on */
	    IPNetwork n( iface->getAddress() , iface->getNetmask() );
	    if ( n.belongs( addr ) ) {
		output << "ifconfig " 
			     << iface->getName() << " "
			     << addr.toString() << " alias" << endl;
		virtual_addresses.push_back(addr);
		return;
	    }
	}
	throw FWException("Can not add virtual address " + addr.toString() );
    }
}

