/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_ipf.cc,v 1.22 2002/05/13 04:13:51 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PolicyCompiler_ipf.hh"

#include "fwbuilder/FWObjectDatabase.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Policy.hh"


using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;

string PolicyCompiler_ipf::myPlatformName() { return "ipf"; }

int PolicyCompiler_ipf::prolog()
{
    if (fw->getStr("platform")!="ipf") 
	abort("Unsupported platform " + fw->getStr("platform") );

    int n= PolicyCompiler::prolog();


    FWObject    *grp;
    anytcp=TCPService::cast(
	dbcopy->create(TCPService::TYPENAME,false) );
    grp=dbcopy->getById( dbcopy->std.TCPServicesId , true );
    assert(grp!=NULL);
    grp->add(anytcp);
    objcache[anytcp->getId()]=anytcp; // to keep cache consistent

    anyudp=UDPService::cast(
	dbcopy->create(UDPService::TYPENAME,false) );
    grp=dbcopy->getById( dbcopy->std.UDPServicesId , true );
    assert(grp!=NULL);
    grp->add(anyudp);
    objcache[anyudp->getId()]=anyudp; // to keep cache consistent

    anyicmp=ICMPService::cast(
	dbcopy->create(ICMPService::TYPENAME,false) );
    grp=dbcopy->getById( dbcopy->std.ICMPServicesId , true );
    assert(grp!=NULL);
    grp->add(anyicmp);
    objcache[anyicmp->getId()]=anyicmp; // to keep cache consistent


    return n;
}

void PolicyCompiler_ipf::expandAnyService::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementSrv *srv=rule->getSrv();
    FWOptions *ruleopt =rule->getOptionsObject();

    if (srv->isAny() && ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept) {

	PolicyRule *r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	RuleElementSrv *nsrv=r->getSrv();
	nsrv->clearChildren();
	nsrv->addRef(getCompiler()->anyicmp);
	vr->push_back(r);

	r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	nsrv=r->getSrv();
	nsrv->clearChildren();
	nsrv->addRef(getCompiler()->anytcp);
	vr->push_back(r);

	r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	nsrv=r->getSrv();
	nsrv->clearChildren();
	nsrv->addRef(getCompiler()->anyudp);
	vr->push_back(r);

	r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,true) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	FWOptions *ruleopt =r->getOptionsObject();
	ruleopt->setBool("stateless",true);
	vr->push_back(r);

    } else
	vr->push_back(rule);
}

void PolicyCompiler_ipf::doSrcNegation::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementSrc *src=rule->getSrc();

    if (src->getNeg()) {
        RuleElementSrc *nsrc;
	PolicyRule     *r;

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setAction(PolicyRule::Continue);
        nsrc=r->getSrc();
        nsrc->setNeg(false);
	r->setBool("quick",false);
	vr->push_back(r);

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
        nsrc=r->getSrc();
        nsrc->setNeg(false);
	nsrc->clearChildren();
	nsrc->setAnyElement();
	r->setBool("quick",true);
	vr->push_back(r);

	return;
    }
    vr->push_back(rule);
}

void PolicyCompiler_ipf::doDstNegation::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementDst *dst=rule->getDst();

    if (dst->getNeg()) {
        RuleElementDst *ndst;
	PolicyRule     *r;

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setAction(PolicyRule::Continue);
        ndst=r->getDst();
        ndst->setNeg(false);
	r->setBool("quick",false);
	vr->push_back(r);

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
        ndst=r->getDst();
        ndst->setNeg(false);
	ndst->clearChildren();
	ndst->setAnyElement();
	r->setBool("quick",true);
	vr->push_back(r);

	return;
    }
    vr->push_back(rule);
}

void PolicyCompiler_ipf::doSrvNegation::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementSrv *srv=rule->getSrv();

    if (srv->getNeg()) {
	throw FWException("Negation in Srv is not implemented. Rule: "+rule->getStr("rule_label"));
	return;
    }
    vr->push_back(rule);
}

void PolicyCompiler_ipf::calculateSkip()
{
    for (list<FWObject*>::iterator k=combined_ruleset->begin(); k!=combined_ruleset->end(); ++k) 
    {
	PolicyRule *r = PolicyRule::cast( *k );
	string rl=r->getStr("rule_label");
	int    current_position=r->getPosition();
	if (r->getAction()==PolicyRule::Continue) {
	    r->setAction(PolicyRule::Skip);

	    list<FWObject*>::iterator j=k;
	    ++j;
	    int n=0;
	    for ( ; j!=combined_ruleset->end(); ++j) {
		PolicyRule *r2 = PolicyRule::cast( *j );
		if (r2->getPosition()!=current_position) break;
		++n;
	    }
	    r->setInt("no_to_skip",n);
	}
    }
}



void PolicyCompiler_ipf::compile()
{
    cout << " Compiling policy for " << fw->getName() << " ..." <<  endl << flush;

    try {

	Compiler::compile();

        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "add default policy rule to the bottom ..\n" << flush;
	addDefaultPolicyRule();
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "verify for MAC address filtering ...\n" << flush;
        for_each_rule( MACFiltering(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "set 'quick' flag ...\n" << flush;
	for_each_rule( setQuickFlag(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "process negation ...\n" << flush;
	for_each_rule( doSrcNegation(this) );
	for_each_rule( doDstNegation(this) );
	for_each_rule( doSrvNegation(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "expand groups ...\n" << flush;
        for_each_rule( ExpandGroups(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rule if firewall is in Src ..\n" << flush;
	for_each_rule( splitIfFirewallInSrc(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rule if firewall is in Dst ..\n" << flush;
	for_each_rule( splitIfFirewallInDst(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "determine directions ..\n" << flush;
	for_each_rule( fillDirection(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rules with direction 'both' ...\n" << flush;
	for_each_rule( SplitDirection(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "expand objects with multiple addresses ...\n" << flush;
	for_each_rule( ExpandMultipleAddresses(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "expand address range objects  ...\n" << flush;
	for_each_rule( addressRanges(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

//	if (verbose) cout << "process negation ...\n" << flush;
//	for_each_rule( doSrcNegation(this) );
//	for_each_rule( doDstNegation(this) );
//	for_each_rule( doSrvNegation(this) );
//      if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rules with different protocols ...\n" << flush;
	for_each_rule( splitServices(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "separate TCP services with flags ...\n" << flush;
	for_each_rule( separateTCPWithFlags(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "verify custom services for this platform ...\n" << flush;
	for_each_rule( verifyCustomServices(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "check for special services ...\n" << flush;
	for_each_rule( SpecialServices(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "expand ANY service for stateful rules ...\n" << flush;
	for_each_rule( expandAnyService(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "convert to atomic rules ...\n" << flush;
        for_each_rule( ConvertToAtomic(this)  );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "calculate argument for skip ...\n" << flush;
	calculateSkip();
        if (debug_rule>=0) for_each_rule( debugRule(this) );

//	if (verbose) cout << "detect rule shading ...\n" << flush;
//	for_each_rule( DetectShading(this) );

    } catch (FWException &ex) {
	abort(ex.toString());
    }
}


void PolicyCompiler_ipf::epilog()
{
    try {
	if (verbose) cout << "printing ... \n" << flush;

	string current_rule_label="";

	for (list<FWObject*>::iterator k=combined_ruleset->begin(); k!=combined_ruleset->end(); ++k) {
	    PolicyRule *r = PolicyRule::cast( *k );
	    string rl=r->getStr("rule_label");
	    if (rl!=current_rule_label) {

                if (verbose) {
                    cout << "Rule " << rl << endl << flush;
                }

		output << "# " << endl;
		output << "# Rule " << rl << endl;

                string    comm=r->getComment();
                string::size_type c1,c2;
                c1=0;
                while ( (c2=comm.find('\n',c1))!=string::npos ) {
                    output << "# " << comm.substr(c1,c2-c1) << endl;
                    c1=c2+1;
                }
                output << "# " << comm.substr(c1) << endl;
                output << "# " << endl;

		current_rule_label=rl;
	    }
	    PrintRule(this)(r);
	}

	cout << " Policy compiled successfully \n" << flush;

    } catch (FWException ex) {
	abort(ex.toString());
    }
}
