/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_ipf.hh,v 1.8 2002/04/11 21:49:29 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef _POLICYCOMPILER_IPF_HH
#define _POLICYCOMPILER_IPF_HH

#include "PolicyCompiler_pf.hh"


namespace libfwbuilder {
class TCPService;
class UDPService;
class ICMPService;
};

namespace fwcompiler {


    class PolicyCompiler_ipf : public PolicyCompiler_pf {

	protected:

	libfwbuilder::TCPService   *anytcp;
	libfwbuilder::UDPService   *anyudp;
	libfwbuilder::ICMPService  *anyicmp;

	virtual std::string _printPort(int rs,int re,bool neg=false);
	virtual void _printWith(libfwbuilder::Service *srv);
	virtual void _printAction(libfwbuilder::PolicyRule *r);
	virtual void _printAddr(libfwbuilder::Address  *o,bool neg=false);

	virtual std::string myPlatformName();

	/**
	 * split rules with service 'any' because ipf can keep state
	 * only for UDP/TCP/ICMP
	 */
	DECLARE_POLICY_RULE_PROCESSOR( PolicyCompiler_ipf, expandAnyService );

	/**
	 *  this inspector prints single policy rule, assuming all
	 *  groups have been expanded, so source, destination and
	 *  service hold exactly one object each, and this object is
	 *  not a group.  Negation should also have been taken care of
	 *  before this method is called.
	 */
 	DECLARE_POLICY_RULE_INSPECTOR( PolicyCompiler_ipf, PrintRule );

        /**
         * this processor deals with negation in Src in policy rules.
	 * 
         * this processor differs from that in PolicyCompiler_pf
         */
        DECLARE_POLICY_RULE_PROCESSOR( PolicyCompiler_ipf, doSrcNegation );

	/**
	 * this processor deals with negation in Dst in policy rules.
	 * 
         * this processor differs from that in PolicyCompiler_pf
	 */
	DECLARE_POLICY_RULE_PROCESSOR( PolicyCompiler_ipf, doDstNegation );

	/**
	 * this processor deals with negation in Srv in policy rules.
	 * 
         * this processor differs from that in PolicyCompiler_pf
	 */
	DECLARE_POLICY_RULE_PROCESSOR( PolicyCompiler_ipf, doSrvNegation );

	/**
	 * calculates N for action skip (used in negation)
	 */
 	void calculateSkip();


	public:

	PolicyCompiler_ipf(libfwbuilder::FWObjectDatabase *_db,
			   const std::string &fwname,
			   fwcompiler::OSConfigurator *_oscnf) : PolicyCompiler_pf(_db,fwname,_oscnf) {}


	virtual int  prolog();
	virtual void compile();
	virtual void epilog();
	
    };


}

#endif
