/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_ipf_writers.cc,v 1.1 2002/04/18 06:39:48 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PolicyCompiler_ipf.hh"

#include "fwbuilder/AddressRange.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/Policy.hh"

#include <iostream.h>
#include <streambuf.h>
#include <iomanip.h>
#include <fstream>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;





/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */


string PolicyCompiler_ipf::_printPort(int rs,int re,bool neg)
{
    ostringstream  str;

    if (rs<0) rs==0;
    if (re<0) re==0;

    if (!neg) {

	if (rs>0 || re>0) {
	    if (rs==re)  str << "= " << rs;
	    else
		if (rs==0 && re!=0)      str << "<= " << re;
		else
		    if (rs!=0 && re==65535)  str << ">= " << rs;
		    else {
/* 
 * port range. Operator '><' defines range in a such way that boundaries
 * are not included. Since we assume it is inclusive, let's move boundaries 
 */
			if (rs>0    ) rs--;
			if (re<65535) re++;
			str << rs << " >< " << re;
		    }
	}
    } else {

	if (rs>0 || re>0) {
	    if (rs==re)  str << "!= " << rs;
	    else
		if (rs==0 && re!=0)      str << "> " << re;
		else
		    if (rs!=0 && re==65535)  str << "< " << rs;
		    else {
			str << rs << " <> " << re;
		    }
	}

    }
    return str.str();
}

void PolicyCompiler_ipf::_printAction(PolicyRule *rule)
{
/*
 * difference between this and PolicyCompiler_pf::_printAction is as follows:
 *
 * 1. there is no support for action Scrub in ipf
 * 2. there is support for return-icmp-as-dest in ipf
 * 3. there is support for action Skip in ipf
 */

    FWOptions *ruleopt =rule->getOptionsObject();
    Service *srv=rule->getSrv()->getFirst(true);    assert(srv);

    switch (rule->getAction()) {
    case PolicyRule::Skip:    output << "skip " << rule->getInt("no_to_skip") << " "; break;
    case PolicyRule::Accept:  output << "pass "; break;
    case PolicyRule::Deny:    output << "block "; break;
    case PolicyRule::Reject: 
	if (TCPService::isA(srv)) output << "block return-rst ";
	else {
	    string aor=ruleopt->getStr("action_on_reject");
	    string code;
	    if ( aor.find("ICMP")!=string::npos ) {
		if (ruleopt->getBool("ipf_return_icmp_as_dest"))
		    code="return-icmp-as-dest ";
		else
		    code="return-icmp ";
		if (aor.find("unreachable")!=string::npos ) {
		    if (aor.find("net")!=string::npos)      code=code+"( 0 ) ";
		    if (aor.find("host")!=string::npos)     code=code+"( 1 ) ";
		    if (aor.find("protocol")!=string::npos) code=code+"( 2 ) ";
		    if (aor.find("port")!=string::npos)     code=code+"( 3 ) ";
		}
		if (aor.find("prohibited")!=string::npos ) {
		    if (aor.find("net")!=string::npos)      code=code+"( 9 ) ";
		    if (aor.find("host")!=string::npos)     code=code+"( 10 ) ";
		}
	    } else
		code="return-icmp   ";

	    output << "block " << code;

	}
	break;
    default:   output << rule->getActionAsString() << " ";
    }
}

void PolicyCompiler_ipf::_printWith(libfwbuilder::Service *srv)
{
    if (IPService::cast(srv)!=NULL) {
	if ( srv->getBool("short_fragm") ) {
	    output << " with short";
	}

	if ( srv->getBool("fragm") ) {
	    output << " with fragm";
	}

	if (srv->getBool("rr")   ) output << " with opt rr";
	if (srv->getBool("ssrr") ) output << " with opt ssrr";
	if (srv->getBool("ts")   ) output << " with opt ts";
    }
}

/*
 *  this is almost like the one in PolicyCompiler_pf, except it does
 *  not print interface name for dynamic interface ('cause ipfilter
 *  does not support it)
 */
void PolicyCompiler_ipf::_printAddr(Address  *o,bool neg)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL) {
	mask=Netmask("255.255.255.255");
    }

    if (addr.toString()=="0.0.0.0" && mask.toString()=="0.0.0.0") {
	output << "any ";
    } else {	
	if (neg) output << "! ";
	output << addr.toString();
	if (mask.toString()!="255.255.255.255") {
	    output << "/" << mask.getLength();
	}
	output << " ";
    }
}


void PolicyCompiler_ipf::PrintRule::operator()(PolicyRule *rule) const
{
    FWOptions *ruleopt =rule->getOptionsObject();

    RuleElementSrc *srcrel=rule->getSrc();
    Address        *src   =srcrel->getFirst(true);  assert(src);
    RuleElementDst *dstrel=rule->getDst();
    Address        *dst   =dstrel->getFirst(true);  assert(dst);
    RuleElementSrv *srvrel=rule->getSrv();
    Service        *srv   =srvrel->getFirst(true);  assert(srv);

    getCompiler()->_printAction(rule);
    getCompiler()->_printDirection(rule);

    if (rule->getLogging()) {
	getCompiler()->output << " log ";
	if (getCompiler()->fwopt->getBool("ipf_log_or_block")) getCompiler()->output << " or-block";
	if (getCompiler()->fwopt->getBool("ipf_log_body"))     getCompiler()->output << " body";
	string facility = getCompiler()->fwopt->getStr("ipf_log_facility");
	string level    = getCompiler()->fwopt->getStr("ipf_log_level");
	if (level!="") {
	    getCompiler()->output << " level ";
	    if (facility!="") getCompiler()->output << facility << ".";
	    getCompiler()->output << level;
	}
    }

    if ( rule->getBool("quick") ) getCompiler()->output << " quick ";
    getCompiler()->_printInterface(rule);
    getCompiler()->_printProtocol(srv);

    getCompiler()->output << " from ";
    getCompiler()->_printSrcAddr(srcrel);
    getCompiler()->_printSrcService(srvrel);

    getCompiler()->output << " to ";
    getCompiler()->_printDstAddr(dstrel);
    getCompiler()->_printDstService(srvrel);

    getCompiler()->_printWith(srv);

/* keeping state does not apply to deny/reject */
    if ( ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept) {  
	getCompiler()->output << "keep state ";
    }

/* keep frags option */
    if ( ruleopt->getBool("ipf_keep_frags") && rule->getAction()==PolicyRule::Accept) {  
	getCompiler()->output << "keep frags ";
    }

    getCompiler()->output << endl;
}


