/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_pf.cc,v 1.31 2002/05/13 04:13:51 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PolicyCompiler_pf.hh"

#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/Service.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/Policy.hh"

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;

string PolicyCompiler_pf::myPlatformName() { return "pf"; }

int PolicyCompiler_pf::prolog()
{


    if (fw->getStr("platform")!="pf") 
	abort("Unsupported platform " + fw->getStr("platform") );

    return PolicyCompiler::prolog();
}

void PolicyCompiler_pf::splitIfFirewallInSrc::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    PolicyRule     *r;
    RuleElementSrc *src=rule->getSrc();    assert(src);

    if (src->size()==1) {
	vr->push_back(rule);
	return;
    }
    FWObject       *fw_in_src=NULL;
    vector<FWObject*> cl;
    for (FWObject::iterator i1=src->begin(); i1!=src->end(); ++i1) {

	FWObject *o   = *i1;
	FWObject *obj = NULL;
//	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
	if (FWReference::cast(o)!=NULL) obj=getCompiler()->objcache[o->getStr("ref")];
	if (obj==NULL)  throw FWException("Broken Src object in rule: "+rule->getStr("rule_label"));

	if (obj->getId()==getCompiler()->fw_id) {
	    fw_in_src=o;   // can not remove right now because remove invalidates iterator

	    RuleElementSrc *nsrc;

	    r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,true) );
	    getCompiler()->temp_ruleset->add(r);
	    r->duplicate(rule);
	    nsrc=r->getSrc();
	    nsrc->clearChildren();
	    nsrc->setAnyElement();
	    nsrc->addRef( getCompiler()->fw );
	    vr->push_back(r);
	}
    }
    if (fw_in_src!=NULL) src->remove( fw_in_src );

    vr->push_back(rule);
}


void PolicyCompiler_pf::splitIfFirewallInDst::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    PolicyRule     *r;
    RuleElementDst *dst=rule->getDst();    assert(dst);

    if (dst->size()==1) {
	vr->push_back(rule);
	return;
    }

    FWObject       *fw_in_dst=NULL;
    vector<FWObject*> cl;
    for (FWObject::iterator i1=dst->begin(); i1!=dst->end(); ++i1) {

	FWObject *o   = *i1;
	FWObject *obj = NULL;
//	if (FWReference::cast(o)!=NULL) obj=FWReference::cast(o)->getPointer();
	if (FWReference::cast(o)!=NULL) obj=getCompiler()->objcache[o->getStr("ref")];
	if (obj==NULL) throw FWException("Broken Dst in rule: "+rule->getStr("rule_label"));

	if (obj->getId()==getCompiler()->fw_id) {
	    fw_in_dst=o;   // can not remove right now because remove invalidates iterator

	    RuleElementDst *ndst;

	    r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,true) );
	    getCompiler()->temp_ruleset->add(r);
	    r->duplicate(rule);
	    ndst=r->getDst();
	    ndst->clearChildren();
	    ndst->setAnyElement();
	    ndst->addRef( getCompiler()->fw );
	    vr->push_back(r);
	}
    }
    if (fw_in_dst!=NULL) dst->remove( fw_in_dst );

    vr->push_back(rule);
}



void PolicyCompiler_pf::fillDirection::operator()(PolicyRule *rule) const
{
    if (rule->getDirectionAsString()=="") {

	if ( getCompiler()->fwopt->getBool("pass_all_out") )  rule->setDirection( PolicyRule::Inbound );
	else {
	    rule->setDirection( PolicyRule::Both );

	    Address  *src=rule->getSrc()->getFirst(true);
	    Address  *dst=rule->getDst()->getFirst(true);
	    string    fwid=getCompiler()->fw_id;

	    if (src->getId()==fwid && dst->getId()==fwid )  return;
	    if (src->getId()==fwid) rule->setDirection( PolicyRule::Outbound );
	    if (dst->getId()==fwid) rule->setDirection( PolicyRule::Inbound );
	}
    }
}


void PolicyCompiler_pf::addDefaultPolicyRule()
{
    if ( fwopt->getBool("pass_all_out") ) {
	PolicyRule *r;

	r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
	temp_ruleset->add(r);
	r->setAction(PolicyRule::Accept);
	r->setLogging(false);
	r->setDirection(PolicyRule::Outbound);
	r->setPosition(10000);
        r->setComment("   fallback rule ");
	combined_ruleset->push_back(r);

	r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
	temp_ruleset->add(r);
	r->setAction(PolicyRule::Deny);
	r->setLogging(false);
	r->setDirection(PolicyRule::Inbound);
	r->setPosition(10001);
        r->setComment("   fallback rule ");
	combined_ruleset->push_back(r);
    } else {
	PolicyRule *r= PolicyRule::cast(dbcopy->create(PolicyRule::TYPENAME,true) );
	temp_ruleset->add(r);
	r->setAction(PolicyRule::Deny);
	r->setLogging(false);
	r->setDirection(PolicyRule::Both);
	r->setPosition(10000);
        r->setComment("   fallback rule ");
	combined_ruleset->push_back(r);
    }
}

void PolicyCompiler_pf::SpecialServices::operator()(PolicyRule *rule) const
{
    RuleElementSrv *srv=rule->getSrv();

    for (FWObject::iterator i=srv->begin(); i!=srv->end(); i++) {
	FWObject *o= *i;
	if (FWReference::cast(o)!=NULL) o=getCompiler()->objcache[o->getStr("ref")];
	Service *s=Service::cast( o );
	assert(s);

	if (IPService::cast(s)!=NULL) {
	    if (s->getBool("rr")        ||
		s->getBool("ssrr")      ||
		s->getBool("ts") )
		rule->setBool("allow_opts",true);
	}
    }
}

void PolicyCompiler_pf::SplitDirection::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    if (rule->getDirection()==PolicyRule::Both) {
	PolicyRule *r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setDirection(PolicyRule::Inbound);
	vr->push_back(r);

	r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setDirection(PolicyRule::Outbound);
	vr->push_back(r);

    } else 
	vr->push_back(rule);
}

void PolicyCompiler_pf::ProcessScrubOption::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    FWOptions *ruleopt =rule->getOptionsObject();


    if ( ruleopt->getBool("scrub") ) {

	if (rule->getAction()!=PolicyRule::Accept) {
	    ruleopt->setBool("scrub",false);
	    vr->push_back(rule);

	    throw FWException("Rule option 'scrub' is supported only for rules with action 'Accept'. Rule: "+rule->getStr("rule_label"));

	    return;
	}

	PolicyRule *r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setAction(PolicyRule::Scrub);
	r->getOptionsObject()->setBool("scrub",false);
	vr->push_back(r);

	ruleopt->setBool("scrub",false);
	vr->push_back(rule);

	return;
    }

/* if service is ip_fragment and action is 'Deny', then add rule with scrub */

    Service *srv=rule->getSrv()->getFirst(true);    assert(srv);

    if ( (srv->getBool("short_fragm") || srv->getBool("fragm")) &&
	 ( rule->getAction()==PolicyRule::Deny || rule->getAction()==PolicyRule::Reject) ) {

	PolicyRule *r= PolicyRule::cast(
	    getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	r->setAction(PolicyRule::Scrub);
	r->getOptionsObject()->setBool("scrub",false);
	vr->push_back(r);

	return;
    }

    vr->push_back(rule);
}

void PolicyCompiler_pf::MACFiltering::operator()(PolicyRule *rule) const
{
    bool src_usemac=false;
    bool dst_usemac=false;

    RuleElementSrc *src=rule->getSrc();
    RuleElementDst *dst=rule->getDst();
    FWOptions *hostopt;

    for (FWObject::iterator i=src->begin(); i!=src->end(); i++) {
	FWObject *o= *i;
	if (FWReference::cast(o)!=NULL) o=getCompiler()->objcache[o->getStr("ref")];
        if (Host::cast(o)!=NULL) {
            hostopt=Host::cast(o)->getOptionsObject();
            src_usemac=hostopt->getBool("use_mac_addr_filter");
        }
    }

    for (FWObject::iterator i=dst->begin(); i!=dst->end(); i++) {
	FWObject *o= *i;
	if (FWReference::cast(o)!=NULL) o=getCompiler()->objcache[o->getStr("ref")];
        if (Host::cast(o)!=NULL) {
            hostopt=Host::cast(o)->getOptionsObject();
            dst_usemac=hostopt->getBool("use_mac_addr_filter");
        }
    }

    if ( src_usemac || dst_usemac )
	throw FWException("MAC address matching is not spupported. Rule: "+rule->getStr("rule_label"));
}


void PolicyCompiler_pf::setQuickFlag::operator()(PolicyRule *rule) const
{
    if ( rule->getAction()!=PolicyRule::Scrub ) rule->setBool("quick",true);
}

void PolicyCompiler_pf::doSrcNegation::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementSrc *src=rule->getSrc();

    if (src->getNeg()) {
        RuleElementSrc *nsrc;
	PolicyRule     *r;

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	if (rule->getAction()==PolicyRule::Accept)  r->setAction(PolicyRule::Deny);
	else	                                    r->setAction(PolicyRule::Accept);
        nsrc=r->getSrc();
        nsrc->setNeg(false);
	r->setBool("quick",true);
        r->setLogging(false);
	vr->push_back(r);

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
        nsrc=r->getSrc();
        nsrc->setNeg(false);
	nsrc->clearChildren();
	nsrc->setAnyElement();
	r->setBool("quick",true);
	vr->push_back(r);

	return;
    }
    vr->push_back(rule);
}

void PolicyCompiler_pf::doDstNegation::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementDst *dst=rule->getDst();

    if (dst->getNeg()) {
        RuleElementDst *ndst;
	PolicyRule     *r;

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
	if (rule->getAction()==PolicyRule::Accept)  r->setAction(PolicyRule::Deny);
	else	                                    r->setAction(PolicyRule::Accept);
        ndst=r->getDst();
        ndst->setNeg(false);
	r->setBool("quick",true);
        r->setLogging(false);
	vr->push_back(r);

	r= PolicyRule::cast(getCompiler()->dbcopy->create(PolicyRule::TYPENAME,false) );
	getCompiler()->temp_ruleset->add(r);
	r->duplicate(rule);
        ndst=r->getDst();
        ndst->setNeg(false);
	ndst->clearChildren();
	ndst->setAnyElement();
	r->setBool("quick",true);
	vr->push_back(r);

	return;
    }
    vr->push_back(rule);
}

void PolicyCompiler_pf::doSrvNegation::operator()(PolicyRule *rule, vector<Rule*> *vr) const
{
    RuleElementSrv *srv=rule->getSrv();

    if (srv->getNeg()) {
	throw FWException("Negation in Srv is not implemented. Rule: "+rule->getStr("rule_label"));
	return;
    }
    vr->push_back(rule);
}
void PolicyCompiler_pf::compile()
{
    cout << " Compiling policy for " << fw->getName() << " ..." <<  endl << flush;

    try {

	Compiler::compile();

        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "add default policy rule to the bottom ..\n" << flush;
	addDefaultPolicyRule();
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "verify for MAC address filtering ...\n" << flush;
        for_each_rule( MACFiltering(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "process negation ...\n" << flush;
	for_each_rule( doSrcNegation(this) );
	for_each_rule( doDstNegation(this) );
	for_each_rule( doSrvNegation(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

        if (verbose) cout << "expand groups ...\n" << flush;
        for_each_rule( ExpandGroups(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rule if firewall is in Src ..\n" << flush;
	for_each_rule( splitIfFirewallInSrc(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rule if firewall is in Dst ..\n" << flush;
	for_each_rule( splitIfFirewallInDst(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "determine directions ..\n" << flush;
	for_each_rule( fillDirection(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rules with direction 'both' ...\n" << flush;
	for_each_rule( SplitDirection(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "expand objects with multiple addresses ...\n" << flush;
	for_each_rule( ExpandMultipleAddresses(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "expand address range objects  ...\n" << flush;
	for_each_rule( addressRanges(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "split rules with different protocols ...\n" << flush;
	for_each_rule( splitServices(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "separate TCP services with flags ...\n" << flush;
	for_each_rule( separateTCPWithFlags(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "verify custom services for this platform ...\n" << flush;
	for_each_rule( verifyCustomServices(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "process 'scrub' option ...\n" << flush;
	for_each_rule( ProcessScrubOption(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "check for special services ...\n" << flush;
	for_each_rule( SpecialServices(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "set 'quick' flag ...\n" << flush;
	for_each_rule( setQuickFlag(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

//	if (verbose) cout << "process negation ...\n" << flush;
//	for_each_rule( doSrcNegation(this) );
//	for_each_rule( doDstNegation(this) );
//	for_each_rule( doSrvNegation(this) );
//      if (debug_rule>=0) for_each_rule( debugRule(this) );

	if (verbose) cout << "detect rule shading ...\n" << flush;
	for_each_rule( DetectShading(this) );
        if (debug_rule>=0) for_each_rule( debugRule(this) );

    } catch (FWException &ex) {
	abort(ex.toString());
    }
}


void PolicyCompiler_pf::epilog()
{
    try {
	if (verbose) cout << "printing ... \n" << flush;

	string current_rule_label="";

	for (list<FWObject*>::iterator k=combined_ruleset->begin(); k!=combined_ruleset->end(); ++k) {
	    PolicyRule *r = PolicyRule::cast( *k );
	    string rl=r->getStr("rule_label");
	    if (rl!=current_rule_label) {

                if (verbose) {
                    cout << rl << endl << flush;
                }

		output << "# " << endl;
		output << "# Rule  " << rl << endl;

                string    comm=r->getComment();
                string::size_type c1,c2;
                c1=0;
                while ( (c2=comm.find('\n',c1))!=string::npos ) {
                    output << "# " << comm.substr(c1,c2-c1) << endl;
                    c1=c2+1;
                }
                output << "# " << comm.substr(c1) << endl;
                output << "# " << endl;

		current_rule_label=rl;
	    }
	    PrintRule(this)(r);
	}

	cout << " Policy compiled successfully \n" << flush;

    } catch (FWException ex) {
	abort(ex.toString());
    }
}
