
/* 

                          Firewall Builder

                 Copyright (C) 2002 Vadim Kurland

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: PolicyCompiler_pf_writers.cc,v 1.2 2002/04/18 22:11:02 vkurland Exp $

  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "PolicyCompiler_pf.hh"

#include "fwbuilder/AddressRange.hh"
#include "fwbuilder/RuleElement.hh"
#include "fwbuilder/IPService.hh"
#include "fwbuilder/ICMPService.hh"
#include "fwbuilder/TCPService.hh"
#include "fwbuilder/UDPService.hh"
#include "fwbuilder/CustomService.hh"
#include "fwbuilder/Policy.hh"

#include <iostream.h>
#include <streambuf.h>
#include <iomanip.h>
#include <fstream>
#include <sstream>

using namespace libfwbuilder;
using namespace fwcompiler;
using namespace std;





/**
 *-----------------------------------------------------------------------
 *                    Methods for printing
 */
void PolicyCompiler_pf::_printAction(PolicyRule *rule)
{
    FWOptions *ruleopt =rule->getOptionsObject();
    Service *srv=rule->getSrv()->getFirst(true);    assert(srv);

    switch (rule->getAction()) {
    case PolicyRule::Accept:  output << "pass "; break;
    case PolicyRule::Deny:    output << "block "; break;
    case PolicyRule::Reject: 
	if (TCPService::isA(srv)) output << "block return-rst ";
	else {
	    string aor=ruleopt->getStr("action_on_reject");
	    string code;
	    if ( aor.find("ICMP")!=string::npos ) {
		code="return-icmp ";
		if (aor.find("unreachable")!=string::npos ) {
 		    if (aor.find("net")!=string::npos)      code=code+"( 0 ) ";
		    if (aor.find("host")!=string::npos)     code=code+"( 1 ) ";
		    if (aor.find("protocol")!=string::npos) code=code+"( 2 ) ";
		    if (aor.find("port")!=string::npos)     code=code+"( 3 ) ";
		}
		if (aor.find("prohibited")!=string::npos ) {
 		    if (aor.find("net")!=string::npos)      code=code+"( 9 ) ";
		    if (aor.find("host")!=string::npos)     code=code+"( 10 ) ";
		}
	    } else
		code="return-icmp   ";

	    output << "block " << code;

	}
	break;
    case PolicyRule::Scrub:   output << "scrub   "; break;
    default:		      output << rule->getActionAsString() << " ";
    }
}

void PolicyCompiler_pf::_printDirection(PolicyRule *rule)
{
    if (rule->getDirection()==PolicyRule::Outbound)  output << "out "; 
    else	                                     output << "in  "; 
}

void PolicyCompiler_pf::_printLogging(PolicyRule *rule)
{
    if (rule->getLogging()) output << " log ";
}

void PolicyCompiler_pf::_printInterface(PolicyRule *rule)
{
    string       iface_id = rule->getStr("interface_id");
    if (iface_id!="") {
	Interface *rule_iface = fw_interfaces[ iface_id ];
	output << "on " << rule_iface->getName() << " ";
    }
}

void PolicyCompiler_pf::_printProtocol(libfwbuilder::Service *srv)
{
    if ( ! srv->isAny() && ! CustomService::isA(srv) && srv->getProtocolName()!="ip") {
	output << "proto ";
	output << srv->getProtocolName();
	output << " ";
    }
}

string PolicyCompiler_pf::_printPort(int rs,int re,bool neg)
{
    ostringstream  str;

    if (rs<0) rs==0;
    if (re<0) re==0;

    if (!neg) {

	if (rs>0 || re>0) {
	    if (rs==re)  str << rs;   // TODO: do we need '=' here ?
	    else
		if (rs==0 && re!=0)      str << "<= " << re;
		else
		    if (rs!=0 && re==65535)  str << ">= " << rs;
		    else {
/* 
 * port range. Operator '><' defines range in a such way that boundaries
 * are not included. Since we assume it is inclusive, let's move boundaries 
 */
			if (rs>0    ) rs--;
			if (re<65535) re++;
			str << rs << " >< " << re;
		    }
	}
    } else {

	if (rs>0 || re>0) {
	    if (rs==re)  str << "!= " << rs;
	    else
		if (rs==0 && re!=0)      str << "> " << re;
		else
		    if (rs!=0 && re==65535)  str << "< " << rs;
		    else {
			str << rs << " <> " << re;
		    }
	}

    }
    return str.str();
}

string PolicyCompiler_pf::_printSrcService(Service *srv,bool neg)
{
    ostringstream  str;
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int rs=srv->getInt("src_range_start");
	int re=srv->getInt("src_range_end");
	str << _printPort(rs,re,neg);
    }
    return str.str();
}

string PolicyCompiler_pf::_printDstService(Service *srv,bool neg)
{
    ostringstream  str;
    if (TCPService::isA(srv) || UDPService::isA(srv)) {
	int rs=srv->getInt("dst_range_start");
	int re=srv->getInt("dst_range_end");
	str << _printPort(rs,re,neg);
    }

    if (ICMPService::isA(srv) && srv->getInt("type")!=-1) {
	str << srv->getStr("type") << " ";
	if (srv->getInt("code")!=-1) 
	    str << "code " << srv->getStr("code") << " ";
    }

    if (CustomService::isA(srv)) {
	str << CustomService::cast(srv)->getCodeForPlatform( myPlatformName() ) << " ";
    }

    return str.str();
}

string PolicyCompiler_pf::_printTCPFlags(libfwbuilder::TCPService *srv)
{
    string str;

    if (srv->getURGFlag()) str+="U";
    if (srv->getACKFlag()) str+="A";
    if (srv->getPSHFlag()) str+="P";
    if (srv->getRSTFlag()) str+="R";
    if (srv->getSYNFlag()) str+="S";
    if (srv->getFINFlag()) str+="F";
    if (!str.empty())      str+="/UAPRSF";
    return str;
}

/*
 * we made sure that all services in rel  represent the same protocol
 */
void PolicyCompiler_pf::_printSrcService(RuleElementSrv  *rel)
{
    Service *srv= rel->getFirst(true);

    if (rel->size()==1) {
	if (UDPService::isA(srv) || TCPService::isA(srv)) {
	    string str=_printSrcService( srv , rel->getNeg());
	    if (! str.empty() ) output << "port " << str << " ";
	}
    } else {

	string str;
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
	    Service *s=Service::cast( o );
	    assert(s);
	    if (UDPService::isA(srv) || TCPService::isA(srv)) {
		string str1= _printSrcService(s , rel->getNeg() );
		if (! str.empty() && ! str1.empty() )  str = str + ", ";
		str = str + str1;
	    }
	}
	if ( !str.empty() ) {
	    output << "port { " << str << "} ";
	}
    }
}

void PolicyCompiler_pf::_printDstService(RuleElementSrv  *rel)
{
    Service *srv= rel->getFirst(true);

    if (rel->size()==1) {
	string str=_printDstService( rel->getFirst(true) , rel->getNeg());
	if ( ! str.empty() ) {
	    if (UDPService::isA(srv) || TCPService::isA(srv)) 
		output << "port " << str << " ";
	    if (ICMPService::isA(srv)) 
		output << "icmp-type " << str << " ";
	}
	if (TCPService::isA(srv)) {
	    str=_printTCPFlags(TCPService::cast(srv));
	    if (!str.empty()) output << "flags " << str << " ";
	}
    } else {

	string str;
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
	    Service *s=Service::cast( o );
	    assert(s);
	    string str1= _printDstService(s , rel->getNeg() );
	    if (! str.empty() && ! str1.empty() )  str = str + ", ";
	    str = str + str1;
	}
	if ( !str.empty() ) {
	    if (UDPService::isA(srv) || TCPService::isA(srv)) 
		output << "port { " << str << " } ";
	    if (ICMPService::isA(srv)) 
		output << "icmp-type { " << str << " } ";
	}
    }
}

void PolicyCompiler_pf::_printAddr(Address  *o,bool neg)
{
    IPAddress addr=o->getAddress();
    Netmask   mask=o->getNetmask();

    if (Interface::cast(o)!=NULL) {
	Interface *interface_=Interface::cast(o);
	if (interface_->isDyn()) {
	    output << interface_->getName() << " ";
	    return;
	}

	mask=Netmask("255.255.255.255");
    }

    if (addr.toString()=="0.0.0.0" && mask.toString()=="0.0.0.0") {
	output << "any ";
    } else {	
	if (neg) output << "! ";
	output << addr.toString();
	if (mask.toString()!="255.255.255.255") {
	    output << "/" << mask.getLength();
	}
	output << " ";
    }
}

void PolicyCompiler_pf::_printSrcAddr(RuleElementSrc  *rel)
{
    if (rel->size()==1) {
	_printAddr( rel->getFirst(true) , rel->getNeg() );
    } else {

	output << "{ ";
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    if (i!=rel->begin())  output << ", ";
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
	    Address *s=Address::cast( o );
	    assert(s);
	    _printAddr(s , rel->getNeg());
	}
	output << "} ";
    }

}

void PolicyCompiler_pf::_printDstAddr(RuleElementDst  *rel)
{
    if (rel->size()==1) {
	_printAddr( rel->getFirst(true) , rel->getNeg());
    } else {

	output << "{ ";
	for (FWObject::iterator i=rel->begin(); i!=rel->end(); i++) {
	    if (i!=rel->begin())  output << ", ";
	    FWObject *o= *i;
//	    if (FWReference::cast(o)!=NULL) o=FWReference::cast(o)->getPointer();
	    if (FWReference::cast(o)!=NULL) o=objcache[o->getStr("ref")];
	    Address *s=Address::cast( o );
	    assert(s);
	    _printAddr(s , rel->getNeg());
	}
	output << "} ";
    }

}

void PolicyCompiler_pf::_printNegation(libfwbuilder::RuleElement  *rel)
{
    if (rel->getNeg())
	output << "! ";
}

void PolicyCompiler_pf::PrintRule::operator()(PolicyRule *rule) const
{
    FWOptions *ruleopt =rule->getOptionsObject();

    RuleElementSrc *srcrel=rule->getSrc();
    Address        *src   =srcrel->getFirst(true);  assert(src);
    RuleElementDst *dstrel=rule->getDst();
    Address        *dst   =dstrel->getFirst(true);  assert(dst);
    RuleElementSrv *srvrel=rule->getSrv();
    Service        *srv   =srvrel->getFirst(true);  assert(srv);

    getCompiler()->_printAction(rule);
    getCompiler()->_printDirection(rule);
    getCompiler()->_printLogging(rule);
    if ( rule->getBool("quick") ) getCompiler()->output << " quick ";
    getCompiler()->_printInterface(rule);
    getCompiler()->_printProtocol(srv);

    getCompiler()->output << " from ";
    getCompiler()->_printSrcAddr(srcrel);
    getCompiler()->_printSrcService(srvrel);

    getCompiler()->output << " to ";
    getCompiler()->_printDstAddr(dstrel);
    getCompiler()->_printDstService(srvrel);

/* keeping state does not apply to deny/reject */
    if ( ! ruleopt->getBool("stateless") && rule->getAction()==PolicyRule::Accept) {  
	if ( getCompiler()->fwopt->getBool("modulate_state") )
	    getCompiler()->output << "modulate state ";
	else
	    getCompiler()->output << "keep state ";
    }

    if (rule->getBool("allow_opts"))
	    getCompiler()->output << "allow-opts  ";

    getCompiler()->output << endl;
}

